from scipy.sparse import csr_array
from .graph import *

class Cycle(Graph):
    r"""
    Cycle graph.

    Parameters
    ----------
    num_vert : int
        Number of vertices in the cycle.

    Notes
    -----
    The cycle can be interpreted as being embedded on the line
    with a cyclic boundary condition.
    In this context,
    we assign the direction ``0`` to the right and ``1`` to the left.
    This assignment alters the order of the arcs.
    Any arc with a tail denoted by :math:`v`
    has the label :math:`2v` if it points to the right,
    and the label :math:`2v + 1` if it points to the left.
    Figure 1 illustrates the arc labels of a cycle with 3 vertices.

    .. graphviz:: ../../graphviz/cycle-arcs.dot
        :align: center
        :layout: neato
        :caption: Figure 1.

    """

    def __init__(self, num_vert):
        # Creating adjacency matrix
        # Every vertex is adjacent to two vertices.
        data = np.ones(2*num_vert, dtype=np.int8)
        # upper diagonal
        row_ind = [lin for lin in range(num_vert)
                       for twice in range(2)]
        # lower digonal
        col_ind = [(col-shift) % num_vert for col in range(num_vert)
                             for shift in [-1, 1]]
        adj_matrix = csr_array((data, (row_ind, col_ind)))
    
        # initializing
        super().__init__(adj_matrix)

    def embeddable(self):
        return True

    def default_coin(self):
        r"""
        Returns the default coin name.
        """
        return 'hadamard'

    def arc_label(self, tail, head):
        num_vert = self.number_of_vertices()
        arc = (2*tail if (head - tail == 1
                          or tail - head == num_vert - 1)
               else 2*tail + 1)
        return arc

    def arc(self, label):
        tail = label//2
        remainder = label % 2
        head = (tail + (-1)**remainder) % self.number_of_vertices()
        return (tail, head)

    def next_arc(self, arc):
        # implemented only if is embeddable
        try:
            tail, head = arc
            num_vert = self.number_of_vertices()
            if (head - tail) % num_vert == 1:
                # clockwise
                return ((tail + 1) % num_vert, (head + 1) % num_vert)
            elif (tail - head) % num_vert == 1:
                # anticlockwise
                return ((tail - 1) % num_vert, (head - 1) % num_vert)
            else:
                raise ValueError('Invalid arc.')
        except TypeError:
            # could not unpack. Arc label passed
            num_arcs = self.number_of_arcs()
            return ((arc + 2) % num_arcs if arc % 2 == 0
                    else (arc - 2) % num_arcs)

    def previous_arc(self, arc):
        # implemented only if is embeddable
        try:
            tail, head = arc
            num_vert = self.number_of_vertices()
            if (head - tail) % num_vert == 1:
                # previous clockwise
                return ((tail - 1) % num_vert, (head - 1) % num_vert)
            elif (tail - head) % num_vert == 1:
                # previous anticlockwise
                return ((tail + 1) % num_vert, (head + 1) % num_vert)
            else:
                raise ValueError('Invalid arc.')
        except TypeError:
            # could not unpack. Arc label passed
            num_arcs = self.number_of_arcs()
            return ((arc - 2) % num_arcs if arc % 2 == 0
                    else (arc + 2) % num_arcs)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
Created on Sat Jan 24 14:29:17 2015

@author: Aaron Leao
         aaron@lncc.br
"""
import sys
import os
import walks
import testmode

tam = len(sys.argv)

if tam>=2:
    string=(str(sys.argv[1]))
    if string =="-test":
        testmode.run()
    
    elif not os.path.isfile( string ):
            print("[Hiperwalk] File '%s' not found."%(string))
            exit(-1)
    else:
        returnValue=walks.walk(string)
        
else:
    print("[Hiperwalk] Missing input file.")
    exit(0)



# -*- coding: utf-8 -*-
"""
Created on Sun Jun  8 22:38:21 2014

@author: aaron

File of functions for calculating STANDARD DEVIATIONS
"""
import numpy as np

import config as cfg

 

def distances_vector_1D(xi,xf):
     cfg.DISTANCES_VECTOR=np.array([np.arange(xi,xf+1,1)])
     cfg.DISTANCES_VECTOR=cfg.DISTANCES_VECTOR.transpose()

 
def generateDistance_CUSTOM():
    fileIn=str(cfg.CUSTOM_LABELS_NAME[0])
    
    qtdeValues = 0  
    for line in open(fileIn,'r'):  
        if line == "\n":  
            continue  
        if line.startswith("#"):  
            continue  
        qtdeValues = qtdeValues + 1

    cfg.GRAPHSIZE=qtdeValues



#            
#def save_XY_Vectors():
#    X = open("X.dat", "w" ) 
#    Y = open("Y.dat", "w" ) 
#    
#    for x in range( cfg.RANGEX[0],cfg.RANGEX[1]+1 ):
#        for y in range( cfg.RANGEY[0],cfg.RANGEY[1]+1 ):
#            X.write("%f\n"%(x))
#            Y.write("%f\n"%(y))
#    X.close()
#    Y.close()        
    
def calculateProbabilities():
    probabilities=np.zeros((cfg.GRAPHSIZE,1),dtype=float)
#    print cfg.COINVECTORDIMENSION
    for i in range(int(cfg.GRAPHSIZE)):
        a=0
        if cfg.WALK=="STAGGERED1D":
            a=cfg.STATE[i]*np.conjugate(cfg.STATE[i])

        else:
#            if cfg.WALK=="DTQW1D":
            for j in range(int(cfg.COINVECTORDIMENSION)):
                a=a+cfg.STATE[(j*cfg.GRAPHSIZE)+i]*np.conjugate(cfg.STATE[(j*cfg.GRAPHSIZE)+i])

        probabilities[i,0]=a.real
    return probabilities
    
    
    
def calculateProbabilities_STAGGERED():
    
    PROBABILITIES=np.zeros((cfg.GRAPHSIZE,1),dtype=float)
    for i in range(int(cfg.GRAPHSIZE)):
        PROBABILITIES[i]= (cfg.STATE[i]).real**2+(cfg.STATE[i]).imag**2
    return PROBABILITIES

def calculateProbabilities_COINED():
    
    PROBABILITIES=np.zeros((cfg.GRAPHSIZE,1),dtype=float)
    for c in range(cfg.COINVECTORDIMENSION):
        for i in range(int(cfg.GRAPHSIZE)):
            PROBABILITIES[i]= PROBABILITIES[i]+(cfg.STATE[c*cfg.GRAPHSIZE+i]).real*(cfg.STATE[c*cfg.GRAPHSIZE+i]).real+(cfg.STATE[c*cfg.GRAPHSIZE+i]).imag*(cfg.STATE[c*cfg.GRAPHSIZE+i]).imag
    return PROBABILITIES



    
def vec_prod(array1,array2):

    returnVector=np.zeros(array1.shape,dtype=float)
    aux=returnVector.shape[0]
    for i in range(aux):
        returnVector[i]=array1[i]*array2[i]
    return returnVector

def vec_sum(array):
    return np.sum(array)
    
def statistics(probabilities,i):
    tmp = vec_prod( cfg.DISTANCES_SQUARE, probabilities )
    
    s1  = vec_sum( tmp )    
    
    tmp = vec_prod( cfg.DISTANCES_VECTOR, probabilities )
    
    mean = vec_sum( tmp )
    
    stddev = np.sqrt(abs(s1-mean*mean))
    return mean,s1,stddev

def calculateProbabilities_COINED2D():
    PROBABILITIES=np.zeros((cfg.GRAPHSIZE,1),dtype=float)
    for c in range(cfg.COINVECTORDIMENSION):
        for i in range(cfg.GRAPHSIZE):
            PROBABILITIES[i]=PROBABILITIES[i]+cfg.STATE[c*cfg.GRAPHSIZE+i].real**2+cfg.STATE[c*cfg.GRAPHSIZE+i].imag**2
    return PROBABILITIES
    
def distances_vector_2D():
     cfg.DISTANCES_VECTOR_X=np.zeros((cfg.GRAPHSIZE,1),float)
     cfg.DISTANCES_VECTOR_Y=np.zeros((cfg.GRAPHSIZE,1),float)
     cfg.DISTANCES_VECTOR_SQUARE_X=np.zeros((cfg.GRAPHSIZE,1),float)
     cfg.DISTANCES_VECTOR_SQUARE_Y=np.zeros((cfg.GRAPHSIZE,1),float)
     
     i=0
     for x in range( cfg.RANGEX[0],cfg.RANGEX[1]+1 ):
         for y in range( cfg.RANGEY[0],cfg.RANGEY[1]+1 ):
            cfg.DISTANCES_VECTOR_X[i]=x
            cfg.DISTANCES_VECTOR_SQUARE_X[i]=x*x
            cfg.DISTANCES_VECTOR_Y[i]=y
            cfg.DISTANCES_VECTOR_SQUARE_Y[i]=y*y
            i=i+1
            
def satistics2D(probabilities):
    aux1=aux2=aux3=aux4=0

    for i in range(cfg.GRAPHSIZE):
        aux1=aux1+cfg.DISTANCES_VECTOR_X[i]*probabilities[i]
        aux2=aux2+cfg.DISTANCES_VECTOR_Y[i]*probabilities[i]
        aux3=aux3+cfg.DISTANCES_VECTOR_SQUARE_X[i]*probabilities[i]
        aux4=aux4+cfg.DISTANCES_VECTOR_SQUARE_Y[i]*probabilities[i]


    
    meanX=aux1
    meanY=aux2
    meanX2=aux3
    meanY2=aux4
    
    varX=meanX2-meanX*meanX
    varY=meanY2-meanY*meanY
    stdv=np.sqrt( varX + varY )


        
    return meanX,meanY,varX,varY,stdv
    



from abc import ABC, abstractmethod
import numpy as np
import scipy.sparse
import inspect
from sys import modules as sys_modules
from .._constants import __DEBUG__, PYNEBLINA_IMPORT_ERROR_MSG
from warnings import warn
from ..graph import Graph
try:
    from . import _pyneblina_interface as nbl
except ModuleNotFoundError:
    warn(PYNEBLINA_IMPORT_ERROR_MSG)

class QuantumWalk(ABC):
    """
    Abstract class for Quantum Walks.

    Basic methods and attributes used for implementing
    specific quantum walk models.

    Parameters
    ----------
    graph
        Graph on which the quantum walk takes place.
        It can be the graph itself (:class:`hiperwalk.graph.Graph`) or
        its adjacency matrix (:class:`scipy.sparse.csr_array`).

    adjacency : :class:`scipy.sparse.csr_array`, optional
        .. deprecated:: 2.0a1
            It will be removed in version 2.0.
            Use ``graph`` instead.

        The adjacency matrix.

    Warns
    -----
    If ``adjacency`` is set. It is deprecated. Use ``graph`` instead.

    Raises
    ------
    TypeError
        if ``adj_matrix`` is not an instance of
        :class:`scipy.sparse.csr_array`.

    Notes
    -----

    .. todo::
        * List the that methods must be overwritten.
        * Accept other types as ``graph`` such as numpy array

    """

    @abstractmethod
    def __init__(self, graph=None, adjacency=None, **kwargs):
        if adjacency is not None:
            if graph is None:
                graph = adjacency
            warn("'adjacency' parameter is deprecated. "
                 + "It will be removed in future versions.")

        if graph is None:
            raise TypeError('graph is None')

        self._marked = (self.set_marked(kwargs['marked'])
                        if 'marked' in kwargs
                        else self.set_marked([]))
        self._evolution = None

        ##############################
        ### Simulation attributes. ###
        ##############################
        # Matrix object used during simulation.
        # It may by a scipy matrix or a neblina matrix.
        # Should be different from None during simulation only.
        self._simul_mat = None
        # Vector object used during simulation.
        # Should be different from None during simulation only.
        self._simul_vec = None


        # TODO: create sparse matrix from graph or dense adjacency matrix
        if isinstance(graph, Graph):
            # DO STUFF
            self._graph = graph

        elif isinstance(graph, scipy.sparse.csr_array):
            if (len(graph.shape) != 2
                or graph.shape[0] != graph.shape[1]
            ):
                raise ValueError(
                    "Adjacency matrix is not a square matrix."
                )

            self._graph = Graph(graph)
        else:
            raise TypeError(
                "Invalid `graph` type."
                + " Expected 'hiperwalk.Graph' or "
                + "'scipy.sparse.csr_array', but received "
                + str(type(adj_matrix)) + ', instead.'
            )

        self.hilb_dim = 0


    def uniform_state(self):
        r"""
        Creates a uniform state.

        The state is constructed based on the ``hilb_dim`` attribute.
        The uniform state is a unit vector with entries 
        that have the same real amplitudes.

        Returns
        -------
        :obj:`numpy.ndarray`

        Notes
        -----
        An example of the uniform state is

        .. math::

            \ket{d} = \frac{1}{\sqrt{N}} \sum_{i = 0}^{N - 1} \ket{i}

        where :math:`N` is the dimension of the Hilbert space.
        """
        return (np.ones(self.hilb_dim, dtype=float)
                / np.sqrt(self.hilb_dim))

    def set_marked(self, marked=[]):
        r"""
        Sets marked vertices.

        Parameters
        ----------
        marked : list of int or int
            List of vertices to be marked.
            If empty list, no vertex is marked.
        """
        if not hasattr(marked, '__iter__'):
            marked = [marked]
        self._marked = list(set(marked))
        self._evolution = None

    def get_marked(self):
        r"""
        Gets marked vertices.

        Returns
        -------
        List of int
            List of marked vertices.
            If no vertex is marked, returns the empty list.
        """
        return list(self._marked)

    @abstractmethod
    def set_evolution(self, **kwargs):
        """
        Create the standard evolution operator.

        The evolution operator is saved to be used during the simulation.

        Parameters
        ----------
        **kwargs : dict, optional
            Additional arguments for constructing the evolution operator

        See Also
        --------
        simulate
        """
        raise NotImplementedError()

    @abstractmethod
    def get_evolution():
        r"""
        Returns the evolution operator in matrix form.
        """
        raise NotImplementedError()

    @staticmethod
    def _elementwise_probability(elem):
        # This is more efficient than:
        # (np.conj(elem) * elem).real
        # elem.real**2 + elem.imag**2
        return elem.real*elem.real + elem.imag*elem.imag

    def probability(self, states):
        r"""
        Computes the probability distribution of each state.

        The probability of each entry of the state.

        Parameters
        ----------
        states : :class:`numpy.ndarray`
            The states used to compute the probabilities.

        Returns
        -------
        probabilities : :class:`numpy.ndarray`
            ``probabilities[i]`` is the probability of the ``i``-entry.

        See Also
        --------
        simulate
        """
        if len(states.shape) == 1:
            states = [states]

        prob = list(map(QuantumWalk._elementwise_probability, states))
        prob = np.array(prob)

        return prob

    def probability_distribution(self, states):
        r"""
        Compute the probability distribution of given states.

        The probability of the walker being found on each vertex
        for the given states.

        Parameters
        ----------
        states : :class:`numpy.ndarray`
            The states used to compute the probabilities.

        Returns
        -------
        probabilities : :class:`numpy.ndarray`
            ``probabilities[i]`` is the probability of the
            walker beign found at vertex ``i``.

        See Also
        --------
        simulate
        """
        return self.probability(states)

    def _time_to_tuple(self, time):
        r"""
        Clean and format ``time`` to ``(start, end, step)`` format.

        See :meth:`simulate` for valid input format options.

        Raises
        ------
        ValueError
            If ``time`` is in an invalid input format.
        """

        if not hasattr(time, '__iter__'):
            time = [time]

        if len(time) == 1:
            start = end = step = time[0]
        elif len(time) == 2:
            start = 0
            end = time[0]
            step = time[1]
        else:
            start = time[0]
            end = time[1]
            step = time[2]
        
        time = [start, end, step]

        if start < 0 or end < 0 or step <= 0:
            raise ValueError(
                "Invalid 'time' value."
                + "'start' and 'end' must be non-negative"
                + " and 'step' must be positive."
            )
        if start > end:
            raise ValueError(
                "Invalid `time` value."
                + "`start` cannot be larger than `end`."
            )

        return time

    def _normalize(self, state, error=1e-16):
        norm = np.linalg.norm(state)
        if 1 - error <= norm and norm <= 1 + error:
            return state
        return state / norm

    def state(self, *args):
        """
        Generates a valid state.

        The state corresponds to the walker being in a superposition
        of the given labels with the given amplitudes.
        The state is normalized in order to be unitary.

        Parameters
        ----------
        *args
            Each entry is a 2-tuple or array with format
            ``(amplitude, vertex)``.

        Returns
        -------
        :class:`numpy.array`

        Examples
        --------
        .. todo::

            valid example
        """
        # TODO benchmark with list comprehension

        # checking if there is a complex entry
        has_complex = np.any([ampl.imag != 0 for ampl, _ in args])
        state = np.zeros(self.hilb_dim,
                         dtype=complex if has_complex else float)
        for ampl, v in args:
            state[v] = ampl

        return self._normalize(state)

    def ket(self, label):
        r"""
        Creates a state of the computational basis.

        Parameters
        ----------
        label : int
            The ket label.

        Examples
        --------
        .. todo::
            valid examples
        """
        ket = np.zeros(self.hilb_dim, dtype=float)
        ket[label] = 1
        return ket

    def _pyneblina_imported(self):
        """
        Expects pyneblina interface to be imported as nbl
        """
        return ('hiperwalk.quantum_walk._pyneblina_interface'
                in sys_modules)

    def simulate(self, time=None, initial_state=None,
                 initial_condition=None, hpc=True):
        r"""
        Simulates the quantum walk.

        Simulates the quantum walk applying the evolution operator
        multiple times to the initial state.

        Parameters
        ----------
        time : int, tuple of int, default=None
            Describes at which time instants the state must be saved.
            It can be specified in three different ways.
            
            * end
                Save the state at time ``end``.
                Only the final state is saved.

            * (end, step)
                Saves each state from time 0 to time ``end`` (inclusive)
                that is multiple of ``step``.

            * (start, end, step)
                Saves every state from time ``start`` (inclusive)
                to time ``end`` (inclusive)
                that is multiple of ``step``.

        initial_state : :class:`numpy.array`, default=None
            The initial state which the evolution operator
            is going to be applied to.

        initial_condition :
            .. deprecated:: 2.0a4
                It will be removed in Hiperwalk 2.0
                due to nomenclature consistency.
                Use ``initial_state`` instead.

        hpc : bool, default=True
            Whether or not to use neblina's high-performance computing
            to perform matrix multiplications.
            If ``hpc=False`` uses standalone python.

        Returns
        -------
        states : :class:`numpy.ndarray`.
            States saved during simulation where
            ``states[i]`` corresponds to the ``i``-th saved state.

        Raises
        ------
        ValueError
            If any of the following occurs
            * ``time=None``.
            * ``initial_state=None``.
            * ``evolution_operator=None`` and it was no set previously.

        See Also
        --------
        evolution_operator
        state

        Notes
        -----
        The walk is simulated by applying the
        evolution operator to the initial state multiple times.
        The maximum and intermediate applications
        are describred by ``time``.

        Examples
        --------
        If ``time=(0, 13, 3)``, the saved states will be:
        the initial state (0), the intermediate states (3, 6, and 9),
        and the final state (12).
        """
        ############################################
        ### Check if simulation was set properly ###
        ############################################
        if time is None:
            raise ValueError(
                "``time` not specified`. "
                + "Must be an int or tuple of int."
            )

        if initial_state is None:
            if initial_condition is None:
                raise ValueError(
                    "``initial_state`` not specified. "
                    + "Expected a np.array."
                )
            else:
                initial_state = initial_condition
                warn('\n`initial_condition` is deprecated. '
                     + 'It will be removed in version 2.0. '
                     + 'Use `initial_state` instead.')

        if len(initial_state) != self.hilb_dim:
            raise ValueError(
                "Initial condition has invalid dimension. "
                + "Expected an np.array with length " + str(self.hilb_dim)
            )

        ###########################
        ### Auxiliary functions ###
        ###########################

        def __prepare_engine(self):
            if hpc:
                self._simul_mat = nbl.send_matrix(self._evolution)
                self._simul_vec = nbl.send_vector(initial_state)

            else:
                self._simul_mat = self._evolution
                self._simul_vec = initial_state

        def __simulate_step(self, step):
            """
            Apply the simulation evolution operator ``step`` times
            to the simulation vector.
            Simulation vector is then updated.
            """
            if hpc:
                # TODO: request multiple multiplications at once
                #       to neblina-core
                # TODO: check if intermediate states are being freed
                for i in range(step):
                    self._simul_vec = nbl.multiply_matrix_vector(
                        self._simul_mat, self._simul_vec)
            else:
                for i in range(step):
                    self._simul_vec = self._simul_mat @ self._simul_vec

                # TODO: compare with numpy.linalg.matrix_power

        def __save_simul_vec(self):
            ret = None

            if hpc:
                # TODO: check if vector must be deleted or
                #       if it can be reused via neblina-core commands.
                ret = nbl.retrieve_vector(self._simul_vec)
            else:
                ret = self._simul_vec

            return ret


        ###############################
        ### simulate implemantation ###
        ###############################

        time = np.array(self._time_to_tuple(time))
        if self._evolution is None:
            self._evolution = self.get_evolution(hpc=hpc)

        if not np.all([e.is_integer() for e in time]):
            raise ValueError("`time` has non-int entry.")

        start, end, step = time

        
        if hpc and not self._pyneblina_imported():
            hpc = False

        __prepare_engine(self)

        # number of states to save
        num_states = int(end/step) + 1
        num_states -= (int((start - 1)/step) + 1) if start > 0 else 0

        # create saved states matrix
        # TODO: error: if initial state is int and
        # evolution operator is float, dtype is complex
        dtype = (initial_state.dtype if
            initial_state.dtype == self._evolution.dtype
            else complex
        )
        saved_states = np.zeros(
            (num_states, initial_state.shape[0]), dtype=dtype
        )
        state_index = 0 # index of the state to be saved

        # if save_initial_state:
        if start == 0:
            saved_states[0] = initial_state.copy()
            state_index += 1
            num_states -= 1

        # simulate walk / apply evolution operator
        if start > 0:
            __simulate_step(self, start - step)

        for i in range(num_states):
            __simulate_step(self, step)
            saved_states[state_index] = __save_simul_vec(self)
            state_index += 1

        # TODO: free vector from neblina core
        self._simul_mat = None
        self._simul_vec = None

        return saved_states

    @staticmethod
    def _get_valid_kwargs(method):
        return inspect.getfullargspec(method)[0][1:]

    @staticmethod
    def _filter_valid_kwargs(kwargs, valid_kwargs):
        return {k : kwargs.get(k) for k in valid_kwargs if k in kwargs}
                #if kwargs.get(k) is not None}

    def hilbert_space_dimension(self):
        """
        Returns dimension of the Hilbert space.
        """
        return self.hilb_dim



import numpy as np
import config as cfg
import ioFunctions as io   
import neblina as nb
import operators as op
import gnuplot as gnuplot
import standardDeviation as sd
import testmode
import os
#from qwalk import test_qwalk as tq



def run():
    probabilities=[]

    if cfg.LATTYPE=="DIAGONAL":
        op.DIAGONAL_SHIFT_OPERATOR_2D()
    elif cfg.LATTYPE=="NATURAL":
        op.NATURAL_SHIFT_OPERATOR_2D()


    op.COIN_TENSOR_IDENTITY_OPERATOR_2D()

    cfg.DISTANCE_VECTOR_SIZE=cfg.GRAPHSIZE
    
    io.savetxt("HIPERWALK_TEMP_PSI.dat",cfg.STATE,float,'%1.16f')

#    nb.generating_DTQW2D_NBL()
    nb.runCore_DTQW2D()
    
    cfg.STATE=nb.neblina_state_to_vector("NEBLINA_TEMP_final_state.dat")
    probabilities=nb.neblina_distribution_to_vector("NEBLINA_TEMP_final_distribution.dat")

    output = open("final_distribution.dat",'w')
    output1 = open("final_state.dat",'w')
    output.write("POS X \t POS Y \t PROBABILITY\n")          
    output1.write("POS X \t POS Y \t COIN \t Re(amplitude)\t \t Im(amplitude)\n")   

#    probabilities=sd.calculateProbabilities_COINED2D()

    
    for x in range(cfg.RANGEX[0],cfg.RANGEX[1]+1,1):
        for y in range(cfg.RANGEY[0],cfg.RANGEY[1]+1,1):
            index=(x-cfg.RANGEX[0])*(cfg.RANGEX[1]-cfg.RANGEX[0]+1)+(y-cfg.RANGEY[0])

            output.write("%d\t %d\t %1.16f\n"%(x,y,probabilities[index]))
            for c in range(cfg.COINVECTORDIMENSION):
                output1.write("%d\t %d\t %d\t%1.16f \t %1.16f\n"%(x,y,c,cfg.STATE[cfg.GRAPHSIZE*c+index].real,cfg.STATE[cfg.GRAPHSIZE*c+index].imag))         
    output.close()
    output1.close()

    if cfg.GNUPLOT:
        io.savetxt("HIPERWALK_TEMP_PROBABILITIES",probabilities,float,'%1.16f')
        gnuplot.template_DTQW2D("HIPERWALK_TEMP_PROBABILITIES","final_distribution.eps","EPS")

        if cfg.STEPS>1:
            gnuplot.plotStatistics2d()
        if cfg.ANIMATION == 1:
            gnuplot.plotAnimation2D()



    if cfg.TEST_MODE:
        modelVector=testmode.create_DTQW2D_test_vector()
        returnNeblina=nb.neblina_distribution_to_vector("NEBLINA_TEMP_final_distribution.dat")
        if np.linalg.norm(modelVector-returnNeblina,np.inf) == float(0):
            return 1
        else:
            return 0
        
    return 1


#import numpy as np
#import networkx as nx
#import sys
#sys.path.append('../..')
#import qwalk as hpw
#
## TODO: random graph
#grid_dim = 5
#grid = nx.grid_graph((grid_dim, grid_dim))
#coined_model = hpw.Coined(nx.adjacency_matrix(grid))
#uniform = coined_model.uniform_state()
#
#num_edges = len(grid.edges())
#print(
#    np.equal(
#        uniform,
#        np.ones(2*num_edges) / np.sqrt(2*num_edges)
#    ).all()
#)



import hiperwalk as hpw

dim = 11
g = hpw.Hypercube(dim)
qw = hpw.CoinedWalk(g)
state = qw.state(*[[1, i] for i in range(dim)])
states = qw.simulate(time=(dim), initial_state=state)
probs = qw.probability_distribution(states)

hpw.plot_probability_distribution(probs, graph=g)



# -*- coding: utf-8 -*-
"""
Created on Mon Jul 21 05:21:53 2014

@author: aaron
"""
import os
import config as cfg



def template_DTQW1D(inputNAME,outputNAME,extension):
    plt_FILE_NAME="HIPERWALK_TEMP_PLOT.plt"
    output = open(plt_FILE_NAME,'w')    

    output.write("reset\n")
    output.write("set grid\n")
    output.write("set xrange[%d:%d]\n"%(cfg.RANGEX[0],cfg.RANGEX[1]))
    output.write("set xlabel \"Position\"\n")
    output.write("set ylabel \"Probability\"\n")

    if cfg.PLOTTING_ZEROS==0:
        output.write("set datafile missing \"0.0000000000000000\"\n")




    if cfg.GRAPHTYPE=="CYCLE":    
        output.write("set title \"Quantum Walk on Cycle\"\n")
    elif cfg.GRAPHTYPE=="LINE":
        output.write("set title \"Quantum Walk on a Line\"\n")

    if extension=="PNG":
        output.write("set terminal png size 1000,500 enhanced font \"Helvetica,20\"\n")
    elif extension=="EPS":    
        output.write("set terminal postscript enhanced eps\n")

    output.write("set output \"%s\"\n"%(outputNAME))
    
    os.system( "paste HIPERWALK_TEMP_RANGE_1D.dat %s > HIPERWALK_TEMP_PDF.dat"%(inputNAME) )

    output.write("plot \"%s\" w l lw 2 lc \"black\" t \"\"\n"%("HIPERWALK_TEMP_PDF.dat"))

    output.close()

    os.system("gnuplot HIPERWALK_TEMP_PLOT.plt 2> /dev/null > /dev/null")


def template_DTQW2D(inputNAME,outputNAME,extension):

    output = open("HIPERWALK_TEMP_PLOT.plt",'w')    

    output.write("reset\n")
    output.write("set border 4095\n")
    
#    output.write("set datafile missing \"0.0000000000000000\"\n")    
    
    output.write("set xrange[%d:%d]\n"%(cfg.RANGEX[0],cfg.RANGEX[1]))
    output.write("set xlabel \"X\"\n")

    output.write("set yrange[%d:%d]\n"%(cfg.RANGEY[0],cfg.RANGEY[1]))
    output.write("set ylabel \"Y\"\n")    

    output.write("set zlabel \"Probability\" rotate by 90 offset -2\n")  

    
    output.write("set xtics offset -1,0\n")
    output.write("set ytics offset 0,-1\n")
    
    output.write("set ticslevel 0\n")

    output.write("set size 1,1\n")
    
    output.write("set view 60,60\n")
    
    output.write("set style data lines\n")
    
    output.write("set dgrid3d %d,%d,16\n"%(cfg.RANGEX[1]-cfg.RANGEX[0]+1,cfg.RANGEY[1]-cfg.RANGEY[0]+1))

    output.write("set isosamples 10000,10000; set samples 100,100;\n")

    if extension=="PNG":
        output.write("set terminal png size 1000,500 enhanced font \"Helvetica,20\"\n")
    elif extension=="EPS":    
        output.write("set terminal postscript color eps\n")
        
    output.write("set output \"%s\"\n"%(outputNAME))

    os.system( "paste HIPERWALK_TEMP_RANGE_2D.dat %s > HIPERWALK_TEMP_PDF"%(inputNAME) )


    output.write("splot \"HIPERWALK_TEMP_PDF\" using 1:2:3 linecolor rgb \"#336699\"  title \"\" with lines\n")    
    output.close()

    os.system("gnuplot HIPERWALK_TEMP_PLOT.plt 2> /dev/null > /dev/null")

def template_STAGGERED1D(inputNAME,outputNAME,extension):
    plt_FILE_NAME="HIPERWALK_TEMP_PLOT.plt"
    output = open(plt_FILE_NAME,'w')    

    output.write("reset\n")
    output.write("set grid\n")
    output.write("set xrange[%d:%d]\n"%(cfg.RANGEX[0],cfg.RANGEX[1]))
    output.write("set xlabel \"Position\"\n")
    output.write("set ylabel \"Probability\"\n")

    if cfg.PLOTTING_ZEROS==0:
        output.write("set datafile missing \"0.0000000000000000\"\n")

    output.write("set title \"Coinless Quantum Walk on Cycle\"\n")


    if extension=="PNG":
        output.write("set terminal png size 1000,500 enhanced font \"Helvetica,20\"\n")
    elif extension=="EPS":    
        output.write("set terminal postscript enhanced eps\n")

    output.write("set output \"%s\"\n"%(outputNAME))
    
    os.system( "paste HIPERWALK_TEMP_RANGE_1D.dat %s > HIPERWALK_TEMP_PDF.dat"%(inputNAME) )

    output.write("plot \"%s\" w l lw 2 lc \"black\" t \"\"\n"%("HIPERWALK_TEMP_PDF.dat"))
    
    output.close()

    os.system("gnuplot HIPERWALK_TEMP_PLOT.plt 2> /dev/null > /dev/null")

#    output.write("set ztics\n")

#    output.write("set mxtics 5\n")
#    output.write("set mytics 5\n")
#    output.write("set hidden3d\n")

#    output.write("set pm3d\n")    
#    output.write("set palette defined (0 \"blue\", 1 \"yellow\", 2 \"red\")\n")
#    output.write("set palette rgbformulae 33,13,10\n")
#    output.write("set palette model RGB\n")    
    
#    output.write("splot \"tmpfile\" using 1:2:3 title \"\" with pm3d\n")
#    output.write("splot \"tmpfile\" using 1:2:3 linecolor palette  title \"\" with lines\n")


 
    
    
def plotAnimation1D():
    print("[HIPERWALK] Creating animation, please wait.")



    for i in range( 0, cfg.STEPS + 1):
        if cfg.ALLSTATES:
            if i % cfg.SAVE_STATES_MULTIPLE_OF_N == 0:
                template_DTQW1D("NEBLINA_TEMP_PROB%05d.dat"%(i),"HIPERWALK_TEMP_PROB%05d.png"%(i),"PNG")
        else:
            template_DTQW1D("NEBLINA_TEMP_PROB%05d.dat"%(i),"HIPERWALK_TEMP_PROB%05d.png"%(i),"PNG")



            
    print( "[HIPERWALK] Generating animation: ................ evolution.gif")
    os.system("convert -delay %d -loop 0 \'HIPERWALK_TEMP_PROB*.png\' HIPERWALK_TEMP.gif"%cfg.DELAY)
    os.system("convert -layers Optimize HIPERWALK_TEMP.gif evolution.gif")  
    
    
def plotAnimation2D():
    print("[HIPERWALK] Creating animation, please wait.")

    for i in range( 0, cfg.STEPS + 1):
        if cfg.ALLSTATES:
            if i % cfg.SAVE_STATES_MULTIPLE_OF_N == 0:
                template_DTQW2D("NEBLINA_TEMP_PROB%05d.dat"%(i),"HIPERWALK_TEMP_PROB%05d.png"%(i),"PNG")
        else:
            template_DTQW2D("NEBLINA_TEMP_PROB%05d.dat"%(i),"HIPERWALK_TEMP_PROB%05d.png"%(i),"PNG")
 
    print( "[HIPERWALK] Generating animation: .................. evolution.gif")
    os.system("convert -delay %d -loop 0 \'HIPERWALK_TEMP_PROB*.png\' HIPERWALK_TEMP_TMP.gif"%cfg.DELAY)
    os.system("convert -layers Optimize HIPERWALK_TEMP_TMP.gif evolution.gif")  
        


###
### Statistical graphics
###

### 2D
def plotStatistics2d():

    print("[HIPERWALK] Generating statistical graphics with gnuplot...")

    generatingStatisticsPlotFile2D("HIPERWALK_TEMP_STANDARD_DEVIATION")
    os.system( "gnuplot HIPERWALK_TEMP_STANDARD_DEVIATION.plt 2> /dev/null > /dev/null")
    print("[HIPERWALK] Standard deviation: .................... standard_deviation.eps")    

    generatingStatisticsPlotFile2D("HIPERWALK_TEMP_MEAN_X")
    os.system( "gnuplot HIPERWALK_TEMP_MEAN_X.plt 2> /dev/null > /dev/null")
    print("[HIPERWALK] Mean(x): ............................... meanX.eps")

    generatingStatisticsPlotFile2D("HIPERWALK_TEMP_MEAN_Y")
    os.system( "gnuplot HIPERWALK_TEMP_MEAN_Y.plt 2> /dev/null > /dev/null")
    print("[HIPERWALK] Mean(y): ............................... meanY.eps")    


    if cfg.VARIANCE:
        generatingStatisticsPlotFile2D("HIPERWALK_TEMP_VARIANCE_X")
        os.system( "gnuplot HIPERWALK_TEMP_VARIANCE_X.plt 2> /dev/null > /dev/null")
        print("[HIPERWALK] Variance(x): ........................... varianceX.eps")    
        
        
        generatingStatisticsPlotFile2D("HIPERWALK_TEMP_VARIANCE_Y")
        os.system( "gnuplot HIPERWALK_TEMP_VARIANCE_Y.plt 2> /dev/null > /dev/null")
        print("[HIPERWALK] Variance(y): ........................... varianceY.eps")    
    
    
def generatingStatisticsPlotFile2D(statisticData):
    output = open("%s.plt"%(statisticData),'w')    
    output.write("reset\n")

    output.write("set fit logfile \'/dev/null\'\n")
    output.write("set term postscript eps enhanced\n")
    output.write("set termoption font \"Helvetica,22\"\n")

    output.write("set xlabel \"Time\"\n")
    output.write("f(x)=a*x+b\n")
    output.write("g(x)= a*x**2 + b*x +c\n")
    
    if( statisticData == "HIPERWALK_TEMP_MEAN_X" ):
        output.write("set title \"Quantum Walk Mean X\"\n")
        output.write("set ylabel \"Mean\"\n")
        output.write("fit f(x) \"./statistics.dat\" u 1:2 via a, b\n")
        output.write("set output \"meanX.eps\"\n")
        output.write("plot \"./statistics.dat\" u 1:2 t \"\", f(x) t sprintf(\"f(x)=ax+b, a=%.8f b=%.8f\", a, b )\n")

        
    elif( statisticData == "HIPERWALK_TEMP_MEAN_Y" ):
        output.write("set title \"Quantum Walk Mean Y\"\n")
        output.write("set ylabel \"Mean\"\n")
        output.write("fit f(x) \"./statistics.dat\" u 1:3 via a, b\n")
        output.write("set output \"meanY.eps\"\n")    
        output.write("plot \"./statistics.dat\" u 1:3 t \"\", f(x) t sprintf(\"f(x)=ax+b, a=%.8f b=%.8f\", a, b )\n")

    elif( statisticData == "HIPERWALK_TEMP_VARIANCE_X" ):
        output.write("set title \"Quantum Walk Variance X\"\n")
        output.write("set ylabel \"Variance\"\n")
        output.write("fit g(x) \"./statistics.dat\" u 1:4 via a, b,c\n")
        output.write("set output \"varianceX.eps\"\n")    
        output.write("plot \"./statistics.dat\" u 1:4 t \"\", g(x) t sprintf(\"a=%.5f b=%.f c=%.5f\", a, b,c)\n")

#        output.write("set title \"Quantum Walk Moment X\"\n")
#        output.write("set ylabel \"Moment\"\n")
#        output.write("fit g(x) \"./statistics.dat\" u 1:7 via a, b,c\n")
#        output.write("set output \"momentX.eps\"\n")    
#        output.write("plot \"./statistics.dat\" u 1:7 t \"\", g(x) t sprintf(\"a=%.5f b=%.f c=%.5f\", a, b,c)\n")



    elif( statisticData == "HIPERWALK_TEMP_VARIANCE_Y" ):
        output.write("set title \"Quantum Walk Variance Y\"\n")
        output.write("set ylabel \"Variance\"\n")
        output.write("fit g(x) \"./statistics.dat\" u 1:5 via a, b,c\n")
        output.write("set output \"varianceY.eps\"\n")    
        output.write("plot \"./statistics.dat\" u 1:5 t \"\", g(x) t sprintf(\"a=%.5f b=%.f c=%.5f\", a, b,c)\n")

#        output.write("set title \"Quantum Walk Moment Y\"\n")
#        output.write("set ylabel \"Moment\"\n")
#        output.write("fit g(x) \"./statistics.dat\" u 1:8 via a, b,c\n")
#        output.write("set output \"momentY.eps\"\n")    
#        output.write("plot \"./statistics.dat\" u 1:8 t \"\", g(x) t sprintf(\"a=%.5f b=%.f c=%.5f\", a, b,c)\n")
    
    elif(statisticData == "HIPERWALK_TEMP_STANDARD_DEVIATION"):
        output.write("set title \"Quantum Walk Standard Deviation\"\n")
        output.write("set ylabel \"Standard Deviation\"\n")
        output.write("fit f(x) \"./statistics.dat\" u 1:6 via a, b\n")
        output.write("set output \"standard_deviation.eps\"\n")
        output.write("plot \"./statistics.dat\" u 1:6 t \"\", f(x) t sprintf(\"f(x)=ax+b, a=%.3f b=%.3f\", a, b )\n")

    output.close()
    

### 1D

def plotStatistics1D():

    print("[HIPERWALK] Generating statistical graphics with gnuplot...")
    generatingStatisticsPlotFile1D("HIPERWALK_TEMP_STANDARD_DEVIATION")
    os.system( "gnuplot HIPERWALK_TEMP_STANDARD_DEVIATION.plt 2> /dev/null > /dev/null")
    print("[HIPERWALK] Standard deviation: ................. standard_deviation.eps")    

    generatingStatisticsPlotFile1D("HIPERWALK_TEMP_MEAN")
    os.system( "gnuplot HIPERWALK_TEMP_MEAN.plt 2> /dev/null > /dev/null")
    print("[HIPERWALK] Mean: ............................... mean.eps")
    
def generatingStatisticsPlotFile1D(statisticData):

    output = open("%s.plt"%statisticData,'w')    
    output.write("reset\n")

    output.write("set fit logfile \'/dev/null\'\n")
    output.write("set term postscript eps enhanced\n")
    output.write("set termoption font \"Helvetica,22\"\n")

    output.write("f(x)=a*x+b\n")

    if( statisticData == "HIPERWALK_TEMP_MEAN" ):
        output.write("fit f(x) \"./statistics.dat\" u 1:2 via a, b\n")
        if cfg.GRAPHTYPE=="CYCLE":    
            output.write("set title \"Quantum Walk Mean on Cycle\"\n")
        elif cfg.GRAPHTYPE=="LINE":
            output.write("set title \"Quantum Walk Mean on a Line\"\n")
        output.write("set output \"mean.eps\"\n")
        output.write("set ylabel \"Mean\"\n")
        output.write("plot \"./statistics.dat\" u 1:2 t \"\", f(x) t sprintf(\"f(x)=ax+b, a=%.3f b=%.3f\", a, b )\n")
    
    elif(statisticData == "HIPERWALK_TEMP_STANDARD_DEVIATION"):
        output.write("fit f(x) \"./statistics.dat\" u 1:4 via a, b\n")
        if cfg.GRAPHTYPE=="CYCLE":    
            output.write("set title \"Quantum Walk Standard Deviation on Cycle\"\n")
        elif cfg.GRAPHTYPE=="LINE":
            output.write("set title \"Quantum Walk Standard Deviation on a Line\"\n")
        output.write("set ylabel \"Standard Deviation\"\n")
        output.write("set output \"standard_deviation.eps\"\n")        
        output.write("plot \"./statistics.dat\" u 1:4 t \"\", f(x) t sprintf(\"f(x)=ax+b, a=%.3f b=%.3f\", a, b )\n")

    output.close()    


# -*- coding: utf-8 -*-
"""
Created on Sun Jul 20 23:25:47 2014

@author: aaron
"""
import numpy as np
import config as cfg


def generateState():

    if cfg.WALK=="DTQW1D":
        return generateState_DTQW1D()

    elif cfg.WALK=="DTQW2D":
        return generateState_DTQW2D()

    elif cfg.WALK=="STAGGERED1D":
        return generateState_STAGGERED1D()

    elif cfg.WALK=="STAGGERED2D":
        return generateState_STAGGERED2D()
        
    elif cfg.WALK=="CUSTOM":
        return generateState_CUSTOM()
        
def generateState_DTQW1D():
    
    if cfg.GRAPHTYPE=="CYCLE":
        rangeX=[int(0),int(cfg.GRAPHSIZE)-1]
        cfg.STATESIZE=cfg.COINVECTORDIMENSION * cfg.GRAPHSIZE

        cfg.STATE=np.zeros((cfg.STATESIZE,1),dtype=complex)
        for i in range(cfg.STATE_COMPONENTS.shape[0]):
            # Find the index of the state.                        
            index=cfg.STATE_COMPONENTS[i][2]*(cfg.GRAPHSIZE)+cfg.STATE_COMPONENTS[i][3]
            # Set the state[index] with the component state[index]=a+bi.
            cfg.STATE[index][0]=cfg.STATE[index][0]+cfg.STATE_COMPONENTS[i][0]+1J*cfg.STATE_COMPONENTS[i][1]
    
    elif cfg.GRAPHTYPE=="LINE":
        upperBound=return_MAX_Position(cfg.STATE_COMPONENTS,3)
        lowerBound=return_MIN_Position(cfg.STATE_COMPONENTS,3)
        rangeX=[int(lowerBound-cfg.STEPS) ,int(upperBound+cfg.STEPS)]
        cfg.GRAPHSIZE=( ( (upperBound-lowerBound) + 1) + 2*cfg.STEPS)
#        print cfg.COINVECTORDIMENSION 
        cfg.STATESIZE=int(cfg.COINVECTORDIMENSION * cfg.GRAPHSIZE)
        cfg.STATE=np.zeros((cfg.STATESIZE,1),dtype=complex)

    
        for i in range(cfg.STATE_COMPONENTS.shape[0]):
            # Find the index of the state.
            index=cfg.STATE_COMPONENTS[i][2]*cfg.GRAPHSIZE + cfg.STEPS + (cfg.STATE_COMPONENTS[i][3]-lowerBound)
            # Set the state[index] with the component state[index]=a+bi.
            cfg.STATE[index][0]=cfg.STATE[index][0]+cfg.STATE_COMPONENTS[i][0]+1J*cfg.STATE_COMPONENTS[i][1]
    cfg.RANGEX=rangeX   

    

def generateState_DTQW2D():



    if cfg.GRAPHTYPE=="TORUS":
        #checkBounderies of TORUS with initialState, if it exceed the size
        checkTorusBoundaries()
        if len(cfg.TORUSSIZE)==1:
            upperBoundX=int(cfg.TORUSSIZE[0])-1
            lowerBoundX=0
            cfg.RANGEX=[int(lowerBoundX) ,int(upperBoundX)]
            cfg.SIZEX=int(upperBoundX)
            upperBoundY=int(cfg.TORUSSIZE[0])-1
            lowerBoundY=0
            cfg.RANGEY=[int(lowerBoundY) ,int(upperBoundY)]
            cfg.SIZEY=int(upperBoundX)
            cfg.GRAPHSIZE=int(cfg.TORUSSIZE[0])*int(cfg.TORUSSIZE[0])
            
        elif len(cfg.TORUSSIZE)==2:
            upperBoundX=int(cfg.TORUSSIZE[0])-1
            lowerBoundX=0
            cfg.RANGEX=[int(lowerBoundX) ,int(upperBoundX)]
            cfg.SIZEX=int(upperBoundX)
            upperBoundY=int(cfg.TORUSSIZE[1])-1
            lowerBoundY=0
            cfg.RANGEY=[int(lowerBoundY) ,int(upperBoundY)]
            cfg.SIZEY=int(upperBoundY)
            cfg.GRAPHSIZE=int(cfg.TORUSSIZE[0])*int(cfg.TORUSSIZE[1])



    
    else:

        upperBoundX=return_MAX_Position(cfg.STATE_COMPONENTS,3)
        lowerBoundX=return_MIN_Position(cfg.STATE_COMPONENTS,3)
        cfg.RANGEX=[int(lowerBoundX-cfg.STEPS) ,int(upperBoundX+cfg.STEPS)]    
        cfg.sizeX=cfg.RANGEX[1]-cfg.RANGEX[0]+1
        
        upperBoundY=return_MAX_Position(cfg.STATE_COMPONENTS,4)
        lowerBoundY=return_MIN_Position(cfg.STATE_COMPONENTS,4)
        cfg.RANGEY=[int(lowerBoundY-cfg.STEPS) ,int(upperBoundY+cfg.STEPS)]
        cfg.sizeY=cfg.RANGEY[1]-cfg.RANGEY[0]+1

        cfg.GRAPHSIZE=cfg.sizeX*cfg.sizeY


#    print cfg.RANGEX
#    print cfg.RANGEY
#    
    cfg.STATESIZE=cfg.GRAPHSIZE*cfg.COINVECTORDIMENSION

    cfg.STATE=np.zeros((cfg.STATESIZE,1),dtype=complex)

    for i in range(cfg.STATE_COMPONENTS.shape[0]):
        # Find the index of the state.
        index=cfg.STATE_COMPONENTS[i][2]*cfg.GRAPHSIZE + (cfg.STATE_COMPONENTS[i][3]-cfg.RANGEX[0])*(cfg.RANGEY[1]-cfg.RANGEY[0]+1)+(cfg.STATE_COMPONENTS[i][4]-cfg.RANGEY[0])
        # Set the state[index] with the component state[index]=a+bi.
        cfg.STATE[index][0]=cfg.STATE[index][0]+cfg.STATE_COMPONENTS[i][0]+1J*cfg.STATE_COMPONENTS[i][1]

#def index():
#    (cfg.RANGEX[1]-cfg.RANGEX[0]+1)*(cfg.STATE_COMPONENTS[i][3]-cfg.RANGEX[0])+(cfg.STATE_COMPONENTS[i][4]-cfg.RANGEY[0])
def generateState_STAGGERED1D():
    
    if cfg.GRAPHTYPE=="CYCLE":
#        if cfg.GRAPHSIZE%2==1:
#            print("STAGGERED Quantum Walk allows only cycles with even sites.")
#            exit(-1)

        aux=cfg.GRAPHSIZE
        aux=np.ceil(aux/cfg.TESSELLATIONPOLYGONS[0])
        aux=aux*cfg.TESSELLATIONPOLYGONS[0]
        cfg.RANGEX=[0,aux-1]
        cfg.TOTAL_PATCHES_IN_X=np.ceil(aux/cfg.TESSELLATIONPOLYGONS[0])
        cfg.STATESIZE=cfg.RANGEX[1]+1
        cfg.STATE=np.zeros((cfg.STATESIZE,1),dtype=complex)
        for i in range(cfg.STATE_COMPONENTS.shape[0]):
            # Find the index of the state.                        
            index=cfg.STATE_COMPONENTS[i][2]
            # Set the state[index] with the component state[index]=a+bi.
            cfg.STATE[index][0]=cfg.STATE[index][0]+cfg.STATE_COMPONENTS[i][0]+1J*cfg.STATE_COMPONENTS[i][1]
    
    elif cfg.GRAPHTYPE=="LINE":



        upperBoundX=return_MAX_Position(cfg.STATE_COMPONENTS,2)
        lowerBoundX=return_MIN_Position(cfg.STATE_COMPONENTS,2)

        auxX0=lowerBoundX-cfg.TESSELLATIONPOLYGONS[0]*cfg.STEPS
        auxX1=upperBoundX+cfg.TESSELLATIONPOLYGONS[0]*cfg.STEPS
     
        auxX3=(auxX1-auxX0)+1
        auxX3=np.ceil(auxX3/cfg.TESSELLATIONPOLYGONS[0])### Roof of the division.
        cfg.TOTAL_PATCHES_IN_X=auxX3
        auxX3=auxX3*cfg.TESSELLATIONPOLYGONS[0]
        
        cfg.RANGEX=[int(auxX0) ,int(auxX0+auxX3-1)]    
        cfg.GRAPHSIZE=cfg.RANGEX[1]-cfg.RANGEX[0]+1

    

            
        cfg.STATESIZE=cfg.GRAPHSIZE
        cfg.STATE=np.zeros((cfg.STATESIZE,1),dtype=complex)

        for i in range(cfg.STATE_COMPONENTS.shape[0]):
            # Find the index of the state.
            index=(cfg.STATE_COMPONENTS[i][2]-cfg.RANGEX[0])
            # Set the state[index] with the component state[index]=a+bi.
            cfg.STATE[index][0]=cfg.STATE[index][0]+cfg.STATE_COMPONENTS[i][0]+1J*cfg.STATE_COMPONENTS[i][1]
        


def generateState_STAGGERED2D():

    if cfg.GRAPHTYPE=="TORUS":
        
        #checkBounderies of TORUS with initialState, if it exceed the size
        checkTorusBoundaries()
        if len(cfg.TORUSSIZE)==1:

            upperBoundX=int(cfg.TORUSSIZE[0])-1
            lowerBoundX=0
            auxX3=(upperBoundX-lowerBoundX)+1
            auxX3=np.ceil(auxX3/cfg.TESSELLATIONPOLYGONS[0])
            cfg.TOTAL_PATCHES_IN_X=auxX3
            cfg.TOTAL_PATCHES_IN_Y=auxX3
            upperBoundX=cfg.TOTAL_PATCHES_IN_X*cfg.TESSELLATIONPOLYGONS[0]
            cfg.RANGEX=[int(lowerBoundX) ,int(upperBoundX)]
            cfg.SIZEX=int(upperBoundX)
            cfg.RANGEY=cfg.RANGEX
            cfg.SIZEY=cfg.SIZEX
            cfg.GRAPHSIZE=int(cfg.cfg.SIZEX*cfg.SIZEY)

            
        elif len(cfg.TORUSSIZE)==2:
           
            upperBoundX=int(cfg.TORUSSIZE[0])-1
            lowerBoundX=0
            cfg.RANGEX=[int(lowerBoundX) ,int(upperBoundX)]
            cfg.SIZEX=int(upperBoundX)
            upperBoundY=int(cfg.TORUSSIZE[1])-1
            lowerBoundY=0
            cfg.RANGEY=[int(lowerBoundY) ,int(upperBoundY)]
            cfg.SIZEY=int(upperBoundY)
            cfg.GRAPHSIZE=int(cfg.TORUSSIZE[0])*int(cfg.TORUSSIZE[1])


    
    elif cfg.GRAPHTYPE=="LATTICE":



        upperBoundX=return_MAX_Position(cfg.STATE_COMPONENTS,2)
        lowerBoundX=return_MIN_Position(cfg.STATE_COMPONENTS,2)
        auxX0=lowerBoundX-cfg.TESSELLATIONPOLYGONS[0]*cfg.STEPS
        auxX1=upperBoundX+cfg.TESSELLATIONPOLYGONS[0]*cfg.STEPS
        auxX3=(auxX1-auxX0)+1
        auxX3=np.ceil(auxX3/cfg.TESSELLATIONPOLYGONS[0])### Roof of the division.
        cfg.TOTAL_PATCHES_IN_X=auxX3
        auxX3=auxX3*cfg.TESSELLATIONPOLYGONS[0]
        
        cfg.RANGEX=[int(auxX0) ,int(auxX0+auxX3-1)]    

        cfg.SIZEX=cfg.RANGEX[1]-cfg.RANGEX[0]+1




        upperBoundY=return_MAX_Position(cfg.STATE_COMPONENTS,3)
        lowerBoundY=return_MIN_Position(cfg.STATE_COMPONENTS,3)        
        auxY0=lowerBoundY-cfg.TESSELLATIONPOLYGONS[1]*cfg.STEPS
        auxY1=upperBoundY+cfg.TESSELLATIONPOLYGONS[1]*cfg.STEPS
        auxY3=(auxY1-auxY0)+1
        auxY3=np.ceil(auxY3/cfg.TESSELLATIONPOLYGONS[1])
        cfg.TOTAL_PATCHES_IN_Y=auxY3
        auxY3=auxY3*cfg.TESSELLATIONPOLYGONS[1]
        cfg.RANGEY=[int(auxY0) ,int(auxY0+auxY3-1)]    
        cfg.SIZEY=cfg.RANGEY[1]-cfg.RANGEY[0]+1

        cfg.GRAPHSIZE=cfg.SIZEX*cfg.SIZEY

    cfg.STATESIZE=cfg.GRAPHSIZE
    cfg.STATE=np.zeros((cfg.STATESIZE,1),dtype=complex)

    for i in range(cfg.STATE_COMPONENTS.shape[0]):
        # Find the index of the state.
        index=(cfg.STATE_COMPONENTS[i][2]-cfg.RANGEX[0])*(cfg.RANGEY[1]-cfg.RANGEY[0]+1)+(cfg.STATE_COMPONENTS[i][3]-cfg.RANGEY[0])
        index=int(index)
        # Set the state[index] with the component state[index]=a+bi.
        cfg.STATE[index][0]=cfg.STATE[index][0]+cfg.STATE_COMPONENTS[i][0]+1J*cfg.STATE_COMPONENTS[i][1]




def generateState_CUSTOM():

    fileIn=str(cfg.CUSTOM_INITIALSTATE_NAME)
    qtdeValues = 0  
    for line in open(fileIn,'r'):  
        if line == "\n":  
            continue  
        if line.startswith("#"):  
            continue  
        qtdeValues = qtdeValues + 1

    cfg.STATESIZE=qtdeValues





#    cfg.STATE=np.zeros((qtdeValues,1),dtype=complex)
#    actual = 0  
#    
#    for line in open(fileIn):  
#        if line == "\n":  
#            continue  
#        if line.startswith("#"):  
#            continue          
#        line = line.split()  
#        a=float(line[0])
#        b=1J*float(line[1])
#
#        cfg.STATE[actual] = a+b
#        actual = actual + 1
#    
#    
#    print cfg.STATE
#


def return_MAX_Position(array,position):
    return max(array[:,position])
    
def return_MIN_Position(array,position):
    return min(array[:,position])
    
    
    
    
def checkTorusBoundaries():

    if cfg.WALK=="DTQW2D":
        auxX=cfg.STATE_COMPONENTS[:,3]
        auxY=cfg.STATE_COMPONENTS[:,4]
    elif cfg.WALK=="STAGGERED2D":

        

        auxX=cfg.STATE_COMPONENTS[:,2]
        auxY=cfg.STATE_COMPONENTS[:,3]

    if  auxX.any()< 0 or auxY.any<0:
        print("[HIPERWALK] GRAPH TORUS only accept positives sites.")
        exit(-1)


    if len(cfg.TORUSSIZE)==1:

        if cfg.WALK=="STAGGERED2D":
            if cfg.TORUSSIZE[0] % cfg.TESSELLATIONPOLYGONS[0] !=0 or cfg.TORUSSIZE[0] % cfg.TESSELLATIONPOLYGONS[1] !=0:
                print("[HIPERWALK] TORUS size must be multiple of number of patches")
                exit(-1)

        for i in auxX:
            if i >= int(cfg.TORUSSIZE[0]):
                print("[HIPERWALK] BEGINSTATE exceed TORUS boundaries.")
                exit(-1)
    
        for i in auxY:
            if i >= int(cfg.TORUSSIZE[0]):
                print("[HIPERWALK] BEGINSTATE exceed TORUS boundaries.")
                exit(-1)

    if len(cfg.TORUSSIZE)==2:

        if cfg.WALK=="STAGGERED2D":
            if cfg.TORUSSIZE[0] % cfg.TESSELLATIONPOLYGONS[0] !=0 or cfg.TORUSSIZE[1] % cfg.TESSELLATIONPOLYGONS[1] !=0:
                print("[HIPERWALK] TORUS size must be multiple of number of patches")
                exit(-1)
        
        for i in auxX:
            if i >= int(cfg.TORUSSIZE[0]):
                print("[HIPERWALK] BEGINSTATE exceed TORUS boundaries.")
                exit(-1)
    
        for i in auxY:
            if i >= int(cfg.TORUSSIZE[1]):
                print("[HIPERWALK] BEGINSTATE exceed TORUS boundaries.")
                exit(-1)
                
                
                
def checkUnitarity(array):
    i=0
    a=0
    while i < len(array):
        a=a+float(array[i])**2+float(array[i+1])**2
        i=i+2

    if abs(a-1.0) > 0.000001:
            print("[HIPERWALK] Error at BEGINTESSELLATION block, superposition is not unitary.")
            exit(-1)



# -*- coding: utf-8 -*-
"""
Created on Sat Jun 14 14:17:41 2014

@author: aaron
"""
import sys
import subprocess

#subprocess.call([sys.executable, 'hiperwalk.py', '-test'])

#subprocess.call([sys.executable, 'hiperwalk.py', 'teste/coined1D.in'])
    #
#subprocess.call([sys.executable, 'hiperwalk.py', 'teste/coined2D.in'])
#
subprocess.call([sys.executable, 'hiperwalk.py', 'teste/staggered1D.in'])
#
#subprocess.call([sys.executable, 'hiperwalk.py', 'teste/staggered2D.in'])
#
#subprocess.call([sys.executable, 'hiperwalk.py', 'teste/custom.in'])




# -*- coding: utf-8 -*-
"""
Created on Mon Jul 21 02:10:29 2014

@author: aaron
"""

import numpy as np
import config as cfg
import ioFunctions as io

#import scipy.sparse as sps


###
### COINS
###
def hadamard(N):
    """ Retorna o operador de Hadamard no formato (1+0j)
    """

    H = np.ones((N, N), dtype=complex)
    for i in range(N):
        for j in range(N):

            a = np.array(map(int, list('{0:010b}'.format(i))))
            b = np.array(map(int, list('{0:010b}'.format(j))))
            #print(a,b,np.dot(a,b))
            H[i][j] = (-1)**np.dot(a, b)

    H = H / np.sqrt(N)
    return H


def fourier(N):
    """ Retorna o operador de fourier para a dimensao N.
    """

    H = np.zeros((N, N), dtype=complex)
    for i in range(N):
        for j in range(N):
            H[i][j] = np.cos((2 * np.pi * i * j) / N) + 1J * np.sin(
                (2 * np.pi * i * j) / N)
    H = H / np.sqrt(N)

    return H


def grover(N):

    G = np.dot((2.0 / N), np.ones((N, N), dtype=complex))

    for i in range(N):
        for j in range(N):
            if i == j:
                G[i][j] = G[i][j] - 1

    return G


def identity(N):
    """ Returns the identity of size N
    """
    return np.eye(N, dtype=complex)


def check_Unitarity(operator, size):
    norm = np.linalg.norm(operator, 2)
    if abs(norm - 1) > 0.00000001:
        print(
            "[HIPERWALK] WARNING: Operator is not unitary, with norm: %f1.16" %
            norm)


##
##  DTQW1D
##
def SHIFT_OPERATOR_1D(COINVECTORSIZE, GRAPHSIZE):
    """ |0><0|TENSOR SOMA(|i+1><i|) + |1><1|TENSOR SOMA(|i-1><i|)
    """
    trunc = int(GRAPHSIZE)

    try:
        f = open("HIPERWALK_TEMP_SHIFT_OPERATOR_1D.dat", 'w')
    except IOError:
        print("[HIPERWALK] Could not open file in directory.")

    for i in range(int(GRAPHSIZE) - 1):
        f.write("%d %d %d %d\n" % (i + 1 + 1, i + 1, 1, 0))
        f.write("%d %d %d %d\n" % (trunc + i + 1, trunc + i + 1 + 1, 1, 0))

    f.write("%d %d %d %d\n" % (1, trunc, 1, 0))
    f.write("%d %d %d %d\n" % (2 * trunc, trunc + 1, 1, 0))


def COIN_TENSOR_IDENTITY_1D(C, sizeI):
    try:
        A = open("HIPERWALK_TEMP_COIN_TENSOR_IDENTITY_1D.dat", 'w')
    except IOError:
        print("[HIPERWALK] Could not open file in directory.")

    for i in range(int(sizeI)):
        A.write("%d %d %1.16f %1.16f\n" %
                (i + 1, i + 1, C[0][0].real, C[0][0].imag))
        A.write("%d %d %1.16f %1.16f\n" %
                (i + 1, i + 1 + sizeI, C[0][1].real, C[0][1].imag))
        A.write("%d %d %1.16f %1.16f\n" %
                (i + 1 + sizeI, i + 1, C[1][0].real, C[1][0].imag))
        A.write("%d %d %1.16f %1.16f\n" %
                (i + 1 + sizeI, i + 1 + sizeI, C[1][1].real, C[1][1].imag))
    A.close()


##
##  DTQW2D
##


def DIAGONAL_SHIFT_OPERATOR_2D():
    try:
        A = open("HIPERWALK_TEMP_SHIFT_OPERATOR_2D.dat", 'w')
    except IOError:
        print("[HIPERWALK] Could not open file in directory.")

    for y in range(cfg.RANGEY[0], cfg.RANGEY[1] + 1, 1):
        for x in range(cfg.RANGEX[0], cfg.RANGEX[1] + 1, 1):

            ###
            ###       S0=|x+1,y+1><x,y|
            ###
            newX = x + 1
            newY = y + 1
            if newX > cfg.RANGEX[1]:
                newX = cfg.RANGEX[0]
            if newY > cfg.RANGEY[1]:
                newY = cfg.RANGEY[0]

            if cfg.GRAPHTYPE == "TORUS":
                indexOld = (x) * (cfg.RANGEY[1] + 1) + (y)
                indexNew = (newX) * (cfg.RANGEY[1] + 1) + (newY)

            else:
                indexOld = (x - cfg.RANGEX[0]) * (
                    cfg.RANGEY[1] - cfg.RANGEY[0] + 1) + (y - cfg.RANGEY[0])
                indexNew = (newX - cfg.RANGEX[0]) * (
                    cfg.RANGEY[1] - cfg.RANGEY[0] + 1) + (newY - cfg.RANGEY[0])
            A.write("%d %d 1 0\n" % (indexNew + 1, indexOld + 1))
            ###
            ###       S1=|x+1,y-1><x,y|
            ###
            newX = x + 1
            newY = y - 1
            if newX > cfg.RANGEX[1]:
                newX = cfg.RANGEX[0]
            if newY < cfg.RANGEY[0]:
                newY = cfg.RANGEY[1]
            if cfg.GRAPHTYPE == "TORUS":
                indexNew = (newX) * (cfg.RANGEY[1] + 1) + (newY)
            else:
                indexNew = (newX - cfg.RANGEX[0]) * (
                    cfg.RANGEY[1] - cfg.RANGEY[0] + 1) + (newY - cfg.RANGEY[0])
            A.write(
                "%d %d 1 0\n" %
                (cfg.GRAPHSIZE + indexNew + 1, cfg.GRAPHSIZE + indexOld + 1))
            ###
            ###       S2=|x-1,y+1><x,y|
            ###
            newX = x - 1
            newY = y + 1
            if newX < cfg.RANGEX[0]:
                newX = cfg.RANGEX[1]
            if newY > cfg.RANGEY[1]:
                newY = cfg.RANGEY[0]
            if cfg.GRAPHTYPE == "TORUS":
                indexNew = (newX) * (cfg.RANGEY[1] + 1) + (newY)
            else:
                indexNew = (newX - cfg.RANGEX[0]) * (
                    cfg.RANGEY[1] - cfg.RANGEY[0] + 1) + (newY - cfg.RANGEY[0])
            A.write("%d %d 1 0\n" % (2 * cfg.GRAPHSIZE + indexNew + 1,
                                     2 * cfg.GRAPHSIZE + indexOld + 1))
            ###
            ###       S3=|x-1,y-1><x,y|
            ###
            newX = x - 1
            newY = y - 1
            if newX < cfg.RANGEX[0]:
                newX = cfg.RANGEX[1]
            if newY < cfg.RANGEY[0]:
                newY = cfg.RANGEY[1]
            if cfg.GRAPHTYPE == "TORUS":
                indexNew = (newX) * (cfg.RANGEY[1] + 1) + (newY)
            else:
                indexNew = (newX - cfg.RANGEX[0]) * (
                    cfg.RANGEY[1] - cfg.RANGEY[0] + 1) + (newY - cfg.RANGEY[0])
            A.write("%d %d 1 0\n" % (3 * cfg.GRAPHSIZE + indexNew + 1,
                                     3 * cfg.GRAPHSIZE + indexOld + 1))
    A.close()


def NATURAL_SHIFT_OPERATOR_2D():
    try:
        A = open("HIPERWALK_TEMP_SHIFT_OPERATOR_2D.dat", 'w')
    except IOError:
        print("[HIPERWALK] Could not open file in directory.")

    for y in range(cfg.RANGEY[0], cfg.RANGEY[1] + 1, 1):
        for x in range(cfg.RANGEX[0], cfg.RANGEX[1] + 1, 1):

            ###
            ###       S0=|x+1,y+1><x,y|
            ###
            newX = x + 0
            newY = y + 1
            if newX > cfg.RANGEX[1]:
                newX = cfg.RANGEX[0]
            if newY > cfg.RANGEY[1]:
                newY = cfg.RANGEY[0]
            if cfg.GRAPHTYPE == "TORUS":
                indexOld = (x) * (cfg.RANGEY[1] + 1) + (y)
                indexNew = (newX) * (cfg.RANGEY[1] + 1) + (newY)

            else:
                indexOld = (x - cfg.RANGEX[0]) * (
                    cfg.RANGEY[1] - cfg.RANGEY[0] + 1) + (y - cfg.RANGEY[0])
                indexNew = (newX - cfg.RANGEX[0]) * (
                    cfg.RANGEY[1] - cfg.RANGEY[0] + 1) + (newY - cfg.RANGEY[0])
            A.write("%d %d 1 0\n" % (0 * cfg.GRAPHSIZE + indexNew + 1,
                                     0 * cfg.GRAPHSIZE + indexOld + 1))
            ###
            ###       S1=|x+1,y-1><x,y|
            ###
            newX = x + 1
            newY = y + 0
            if newX > cfg.RANGEX[1]:
                newX = cfg.RANGEX[0]
            if newY < cfg.RANGEY[0]:
                newY = cfg.RANGEY[1]
            if cfg.GRAPHTYPE == "TORUS":
                indexNew = (newX) * (cfg.RANGEY[1] + 1) + (newY)
            else:
                indexNew = (newX - cfg.RANGEX[0]) * (
                    cfg.RANGEY[1] - cfg.RANGEY[0] + 1) + (newY - cfg.RANGEY[0])
            A.write("%d %d 1 0\n" % (1 * cfg.GRAPHSIZE + indexNew + 1,
                                     1 * cfg.GRAPHSIZE + indexOld + 1))
            ###
            ###       S2=|x-1,y+1><x,y|
            ###
            newX = x - 1
            newY = y + 0
            if newX < cfg.RANGEX[0]:
                newX = cfg.RANGEX[1]
            if newY > cfg.RANGEY[1]:
                newY = cfg.RANGEY[0]
            if cfg.GRAPHTYPE == "TORUS":
                indexNew = (newX) * (cfg.RANGEY[1] + 1) + (newY)
            else:
                indexNew = (newX - cfg.RANGEX[0]) * (
                    cfg.RANGEY[1] - cfg.RANGEY[0] + 1) + (newY - cfg.RANGEY[0])
            A.write("%d %d 1 0\n" % (2 * cfg.GRAPHSIZE + indexNew + 1,
                                     2 * cfg.GRAPHSIZE + indexOld + 1))
            ###
            ###       S3=|x-1,y-1><x,y|
            ###
            newX = x + 0
            newY = y - 1
            if newX < cfg.RANGEX[0]:
                newX = cfg.RANGEX[1]
            if newY < cfg.RANGEY[0]:
                newY = cfg.RANGEY[1]
            if cfg.GRAPHTYPE == "TORUS":
                indexNew = (newX) * (cfg.RANGEY[1] + 1) + (newY)
            else:
                indexNew = (newX - cfg.RANGEX[0]) * (
                    cfg.RANGEY[1] - cfg.RANGEY[0] + 1) + (newY - cfg.RANGEY[0])
            A.write("%d %d 1 0\n" % (3 * cfg.GRAPHSIZE + indexNew + 1,
                                     3 * cfg.GRAPHSIZE + indexOld + 1))

    A.close()


def NOVO_SHIFT_OPERATOR_2D():
    try:
        A = open("HIPERWALK_TEMP_SHIFT_OPERATOR_2D.dat", 'w')
    except IOError:
        print("[HIPERWALK] Could not open file in directory.")

    for y in range(cfg.RANGEY[0], cfg.RANGEY[1] + 1, 1):
        for x in range(cfg.RANGEX[0], cfg.RANGEX[1] + 1, 1):

            ###
            ###       S0=|x+1,y+1><x,y|
            ###
            newX = x + 1
            newY = y + 0

            if newX > cfg.RANGEX[1]:
                newX = cfg.RANGEX[0]
            indexOld = (x - cfg.RANGEX[0]) * (cfg.RANGEY[1] - cfg.RANGEY[0] +
                                              1) + (y - cfg.RANGEY[0])
            indexNew = (newX - cfg.RANGEX[0]) * (
                cfg.RANGEY[1] - cfg.RANGEY[0] + 1) + (newY - cfg.RANGEY[0])
            A.write("%d %d 1 0\n" % (0 * cfg.GRAPHSIZE + indexNew + 1,
                                     0 * cfg.GRAPHSIZE + indexOld + 1))

            ###
            ###       S1=|x+1,y-1><x,y|
            ###
            newX = x - 1
            newY = y + 0
            if newX < cfg.RANGEX[0]:
                newX = cfg.RANGEX[1]
            indexNew = (newX - cfg.RANGEX[0]) * (
                cfg.RANGEY[1] - cfg.RANGEY[0] + 1) + (newY - cfg.RANGEY[0])
            A.write("%d %d 1 0\n" % (1 * cfg.GRAPHSIZE + indexNew + 1,
                                     1 * cfg.GRAPHSIZE + indexOld + 1))

            ###
            ###       S2=|x-1,y+1><x,y|
            ###
            newX = x + 0
            newY = y + 1
            if newY > cfg.RANGEY[1]:
                newY = cfg.RANGEY[0]
            indexNew = (newX - cfg.RANGEX[0]) * (
                cfg.RANGEY[1] - cfg.RANGEY[0] + 1) + (newY - cfg.RANGEY[0])
            A.write("%d %d 1 0\n" % (2 * cfg.GRAPHSIZE + indexNew + 1,
                                     2 * cfg.GRAPHSIZE + indexOld + 1))

            ###
            ###       S3=|x-1,y-1><x,y|
            ###
            newX = x + 0
            newY = y - 1
            if newY < cfg.RANGEY[0]:
                newY = cfg.RANGEY[1]
            indexNew = (newX - cfg.RANGEX[0]) * (
                cfg.RANGEY[1] - cfg.RANGEY[0] + 1) + (newY - cfg.RANGEY[0])
            A.write("%d %d 1 0\n" % (3 * cfg.GRAPHSIZE + indexNew + 1,
                                     3 * cfg.GRAPHSIZE + indexOld + 1))

    A.close()


def COIN_TENSOR_IDENTITY_OPERATOR_2D():
    try:
        f = open("HIPERWALK_TEMP_COIN_TENSOR_IDENTITY_OPERATOR_2D.dat", "w")
    except IOError:
        print("[HIPERWALK] Could not open file in directory.")

    for i in range(4):
        for j in range(4):
            for w in range(cfg.GRAPHSIZE):
                f.write(
                    "%d %d %1.16f %1.16f\n" %
                    (cfg.GRAPHSIZE * i + w + 1, cfg.GRAPHSIZE * j + w + 1,
                     cfg.COINOPERATOR[i][j].real, cfg.COINOPERATOR[i][j].imag))
    f.close()


##
##  STAGGERED
##
def STAGGERED1D():
    #    STAGGERED_EVEN_OPERATOR_1D()
    #    STAGGERED_ODD_OPERATOR_1D()
    teste_STAGGERED_EVEN_OPERATOR_1D()
    teste_STAGGERED_ODD_OPERATOR_1D()


def STAGGERED_EVEN_OPERATOR_1D():
    try:
        f = open("HIPERWALK_TEMP_STAGGERED_EVEN_OPERATOR_1D.dat", 'w')
    except IOError:
        print("[HIPERWALK] Could not open file in directory.")

    alpha = float(cfg.STAGGERED_COEFICIENTS[0][0])
    phi = float(cfg.STAGGERED_COEFICIENTS[0][1])
    aux = np.zeros([2, 2], dtype=complex)
    #    aux[0][0]=np.cos(alpha)
    #    aux[0][1]=np.sin(alpha)*(np.cos(phi)+1J*np.sin(phi))
    #    aux[1][0]=aux[0][1]
    #    aux[1][1]= 2 * np.sin(alpha/2) * np.sin(alpha/2) * ( np.cos(2*phi)+1J*np.sin(2*phi))-1

    aux[0][0] = 2 * np.cos(alpha / 2) * np.cos(alpha / 2) - 1
    aux[0][1] = 2 * np.cos(alpha / 2) * np.sin(
        alpha / 2) * (np.cos(phi) - 1j * np.sin(phi))
    aux[1][0] = 2 * np.cos(alpha / 2) * np.sin(
        alpha / 2) * (np.cos(phi) + 1j * np.sin(phi))
    aux[1][1] = 2 * np.sin(alpha / 2) * np.sin(alpha / 2) - 1

    for i in np.arange(0, cfg.STATE.shape[0] - 1, 2):
        f.write("%d %d %1.16f %1.16f\n" %
                (i + 1, i + 1, aux[0][0].real, aux[0][0].imag))
        f.write("%d %d %1.16f %1.16f\n" %
                (i + 2, i + 1, aux[1][0].real, aux[1][0].imag))
        f.write("%d %d %1.16f %1.16f\n" %
                (i + 1, i + 2, aux[0][1].real, aux[0][1].imag))
        f.write("%d %d %1.16f %1.16f\n" %
                (i + 2, i + 2, aux[1][1].real, aux[1][1].imag))
    f.close()


def STAGGERED_ODD_OPERATOR_1D():
    try:
        f = open("HIPERWALK_TEMP_STAGGERED_ODD_OPERATOR_1D.dat", 'w')
    except IOError:
        print("[HIPERWALK] Could not open file in directory.")

    beta = float(cfg.STAGGERED_COEFICIENTS[1][0])
    phi = float(cfg.STAGGERED_COEFICIENTS[1][1])
    aux = np.zeros([2, 2], dtype=complex)
    #    aux[0][0]=np.cos(beta)
    #    aux[0][1]=np.sin(beta)*(np.cos(phi)+1J*np.sin(phi))
    #    aux[1][0]=aux[0][1]
    #    aux[1][1]=2*np.sin(beta/2)*np.sin(beta/2)*(np.cos(2*phi)+1J*np.sin(2*phi))-1

    aux[0][0] = 2 * np.sin(beta / 2) * np.sin(beta / 2) - 1
    aux[0][1] = 2 * np.sin(beta / 2) * np.cos(
        beta / 2) * (np.cos(phi) - 1J * np.sin(phi))
    aux[1][0] = 2 * np.sin(beta / 2) * np.cos(
        beta / 2) * (np.cos(phi) + 1J * np.sin(phi))
    aux[1][1] = 2 * np.cos(beta / 2) * np.cos(beta / 2) - 1

    f.write("%d %d %1.16f %1.16f\n" % (1, 1, aux[1][1].real, aux[1][1].imag))
    f.write("%d %d %1.16f %1.16f\n" %
            (1, cfg.STATE.shape[0], aux[0][1].real, aux[0][1].imag))
    f.write("%d %d %1.16f %1.16f\n" %
            (cfg.STATE.shape[0], 1, aux[1][0].real, aux[1][0].imag))
    f.write("%d %d %1.16f %1.16f\n" % (cfg.STATE.shape[0], cfg.STATE.shape[0],
                                       aux[0][0].real, aux[0][0].imag))

    for i in np.arange(1, cfg.STATE.shape[0] - 2, 2):
        f.write("%d %d %1.16f %1.16f\n" %
                (i + 1, i + 1, aux[0][0].real, aux[0][0].imag))
        f.write("%d %d %1.16f %1.16f\n" %
                (i + 2, i + 1, aux[1][0].real, aux[1][0].imag))
        f.write("%d %d %1.16f %1.16f\n" %
                (i + 1, i + 2, aux[0][1].real, aux[0][1].imag))
        f.write("%d %d %1.16f %1.16f\n" %
                (i + 2, i + 2, aux[1][1].real, aux[1][1].imag))
    f.close()


def teste_STAGGERED_EVEN_OPERATOR_1D():
    try:
        f = open("HIPERWALK_TEMP_STAGGERED_EVEN_OPERATOR_1D.dat", 'w')
    except IOError:
        print("[HIPERWALK] Could not open file in directory.")

    totalVerticesPerPatch = cfg.TESSELLATIONPOLYGONS[0]
    vertices_in_X = cfg.TESSELLATIONPOLYGONS[0]
    array_Index = np.zeros((totalVerticesPerPatch, 1), dtype=int)
    array_Values = np.zeros((totalVerticesPerPatch, 1), dtype=complex)

    #    cfg.Ueven=np.zeros((cfg.STATESIZE,cfg.STATESIZE),dtype=float)

    counter = 0
    for i in range(int(
            cfg.TOTAL_PATCHES_IN_X)):  ### Run in all patches in axis X

        counter = 0

        for x in range(
                vertices_in_X):  ### Run in all vertices per patches in axis X
            #                auxX=cfg.OVERLAPX*i+x+cfg.RANGEX[0]
            auxX = cfg.OVERLAPX * i + x + cfg.RANGEX[0]
            if cfg.GRAPHTYPE == "CYCLE":
                index = (auxX)
            elif cfg.GRAPHTYPE == "LINE":
                index = (auxX - cfg.RANGEX[0])

            array_Index[counter] = index
            counter = counter + 1

        counter = 0
        for k in range(0, cfg.STAGGERED_COEFICIENTS.shape[1], 2):
            array_Values[counter] = cfg.STAGGERED_COEFICIENTS[0][
                k] + 1J * cfg.STAGGERED_COEFICIENTS[0][k + 1]
            counter = counter + 1

        for i1 in range(totalVerticesPerPatch):
            for j1 in range(totalVerticesPerPatch):
                value = 2 * array_Values[i1] * np.conj(array_Values[j1])
                if (array_Index[i1] == array_Index[j1]):
                    value = value - 1
                f.write("%d %d %1.16f %1.16f\n" %
                        (array_Index[i1] + 1, array_Index[j1] + 1, value.real,
                         value.imag))


#
#                i2=array_Index[i1]
#                j2=array_Index[j1]
#                cfg.Ueven[i2,j2]=value
    f.close()


def teste_STAGGERED_ODD_OPERATOR_1D():
    try:
        f = open("HIPERWALK_TEMP_STAGGERED_ODD_OPERATOR_1D.dat", 'w')
    except IOError:
        print("[HIPERWALK] Could not open file in directory.")

    totalVerticesPerPatch = cfg.TESSELLATIONPOLYGONS[0]

    array_Index = np.zeros((totalVerticesPerPatch, 1), dtype=int)
    array_Values = np.zeros((totalVerticesPerPatch, 1), dtype=complex)

    #    cfg.Uodd=np.zeros((cfg.STATESIZE,cfg.STATESIZE),dtype=float)

    counter = 0

    #    print cfg.TESSELLATIONDISPLACEMENT

    for i in range(int(
            cfg.TOTAL_PATCHES_IN_X)):  ### Run in all patches in axis X
        counter = 0
        for x in range(int(cfg.TESSELLATIONPOLYGONS[0])
                       ):  ### Run in all vertices per patches in axis X
            auxX = cfg.OVERLAPX * i + x + cfg.RANGEX[
                0] + cfg.TESSELLATIONDISPLACEMENT[0]

            if cfg.GRAPHTYPE == "CYCLE":
                if auxX > cfg.RANGEX[1]:
                    auxX = cfg.RANGEX[0]
                index = (auxX)
            elif cfg.GRAPHTYPE == "LINE":
                if auxX > cfg.RANGEX[1]:
                    auxX = cfg.RANGEX[0]
                index = (auxX - cfg.RANGEX[0])

            array_Index[counter] = index
            counter = counter + 1
        counter = 0
        for k in range(0, cfg.STAGGERED_COEFICIENTS.shape[1], 2):
            array_Values[counter] = cfg.STAGGERED_COEFICIENTS[1][
                k] + 1J * cfg.STAGGERED_COEFICIENTS[1][k + 1]
            counter = counter + 1

        for i1 in range(totalVerticesPerPatch):
            for j1 in range(totalVerticesPerPatch):
                value = 2 * array_Values[i1] * np.conj(array_Values[j1])
                if (array_Index[i1] == array_Index[j1]):
                    value = value - 1
                f.write("%d %d %1.16f %1.16f\n" %
                        (array_Index[i1] + 1, array_Index[j1] + 1, value.real,
                         value.imag))

#                    i2=array_Index[i1]
#                    j2=array_Index[j1]
#                    cfg.Uodd[i2,j2]=value
    f.close()


###
### 2D
###
def STAGGERED2D():

    STAGGERED_EVEN_OPERATOR_2D()
    STAGGERED_ODD_OPERATOR_2D()


def STAGGERED_EVEN_OPERATOR_2D():
    f = open("HIPERWALK_TEMP_STAGGERED_EVEN_OPERATOR_2D.dat", 'w')

    totalVerticesPerPatch = cfg.TESSELLATIONPOLYGONS[
        0] * cfg.TESSELLATIONPOLYGONS[1]
    array_Index = np.zeros((totalVerticesPerPatch, 1), dtype=int)
    array_Values = np.zeros((totalVerticesPerPatch, 1), dtype=complex)

    index = 0

    #    cfg.Ueven=np.zeros((cfg.STATESIZE,cfg.STATESIZE),dtype=float)

    counter = 0
    for i in range(int(
            cfg.TOTAL_PATCHES_IN_X)):  ### Run in all patches in axis X
        for j in range(int(
                cfg.TOTAL_PATCHES_IN_Y)):  ### Run in all patches in axis Y
            counter = 0

            for x in range(int(cfg.TESSELLATIONPOLYGONS[0])
                           ):  ### Run in all vertices per patches in axis X
                for y in range(
                        int(cfg.TESSELLATIONPOLYGONS[1]
                            )):  ### Run in all vertices per patches in axis Y

                    auxX = cfg.OVERLAPX * i + x + cfg.RANGEX[0]
                    auxY = cfg.OVERLAPY * j + y + cfg.RANGEY[0]

                    if cfg.GRAPHTYPE == "TORUS":
                        index = (auxX) * (cfg.RANGEY[1] - cfg.RANGEY[0]) + (
                            auxY)
                    elif cfg.GRAPHTYPE == "LATTICE":
                        index = (auxX - cfg.RANGEX[0]) * (
                            cfg.RANGEY[1] - cfg.RANGEY[0] + 1) + (
                                auxY - cfg.RANGEY[0])

#                    print i,j,auxX,auxY,index

                    array_Index[counter] = index
                    counter = counter + 1

            counter = 0
            for k in range(0, cfg.STAGGERED_COEFICIENTS.shape[1], 2):
                array_Values[counter] = cfg.STAGGERED_COEFICIENTS[0][
                    k] + 1J * cfg.STAGGERED_COEFICIENTS[0][k + 1]
                counter = counter + 1

            for i1 in range(totalVerticesPerPatch):
                for j1 in range(totalVerticesPerPatch):
                    value = 2 * array_Values[i1] * np.conj(array_Values[j1])
                    if (array_Index[i1] == array_Index[j1]):
                        value = value - 1
                    f.write("%d %d %1.16f %1.16f\n" %
                            (array_Index[i1] + 1, array_Index[j1] + 1,
                             value.real, value.imag))


#
#                    i2=array_Index[i1]
#                    j2=array_Index[j1]
#                    cfg.Ueven[i2,j2]=value
    f.close()


def STAGGERED_ODD_OPERATOR_2D():
    f = open("HIPERWALK_TEMP_STAGGERED_ODD_OPERATOR_2D.dat", 'w')

    totalVerticesPerPatch = cfg.TESSELLATIONPOLYGONS[
        0] * cfg.TESSELLATIONPOLYGONS[1]
    array_Index = np.zeros((totalVerticesPerPatch, 1), dtype=int)
    array_Values = np.zeros((totalVerticesPerPatch, 1), dtype=complex)

    #    cfg.Uodd=np.zeros((cfg.STATESIZE,cfg.STATESIZE),dtype=float)

    index = 0
    counter = 0
    for i in range(int(
            cfg.TOTAL_PATCHES_IN_X)):  ### Run in all patches in axis X
        for j in range(int(
                cfg.TOTAL_PATCHES_IN_Y)):  ### Run in all patches in axis Y
            counter = 0
            for x in range(int(cfg.TESSELLATIONPOLYGONS[0])
                           ):  ### Run in all vertices per patches in axis X
                for y in range(
                        int(cfg.TESSELLATIONPOLYGONS[1]
                            )):  ### Run in all vertices per patches in axis Y
                    auxX = cfg.OVERLAPX * i + x + cfg.RANGEX[
                        0] + cfg.TESSELLATIONDISPLACEMENT[0]
                    auxY = cfg.OVERLAPY * j + y + cfg.RANGEY[
                        0] + cfg.TESSELLATIONDISPLACEMENT[1]

                    if cfg.GRAPHTYPE == "TORUS":
                        if auxX == cfg.RANGEX[1]:
                            auxX = cfg.RANGEX[0]
                        if auxY == cfg.RANGEY[1]:
                            auxY = cfg.RANGEY[0]
                        index = (auxX) * (cfg.RANGEY[1] - cfg.RANGEY[0]) + (
                            auxY)
                    elif cfg.GRAPHTYPE == "LATTICE":
                        if auxX > cfg.RANGEX[1]:
                            auxX = cfg.RANGEX[0]
                        if auxY > cfg.RANGEY[1]:
                            auxY = cfg.RANGEY[0]
                        index = (auxX - cfg.RANGEX[0]) * (
                            cfg.RANGEY[1] - cfg.RANGEY[0] + 1) + (
                                auxY - cfg.RANGEY[0])

                    array_Index[counter] = index
                    counter = counter + 1
            counter = 0
            for k in range(0, cfg.STAGGERED_COEFICIENTS.shape[1], 2):
                array_Values[counter] = cfg.STAGGERED_COEFICIENTS[1][
                    k] + 1J * cfg.STAGGERED_COEFICIENTS[1][k + 1]
                counter = counter + 1

            for i1 in range(totalVerticesPerPatch):
                for j1 in range(totalVerticesPerPatch):
                    value = 2 * array_Values[i1] * np.conj(array_Values[j1])
                    if (array_Index[i1] == array_Index[j1]):
                        value = value - 1
                    f.write("%d %d %1.16f %1.16f\n" %
                            (array_Index[i1] + 1, array_Index[j1] + 1,
                             value.real, value.imag))


#                    i2=array_Index[i1]
#                    j2=array_Index[j1]
#                    cfg.Uodd[i2,j2]=value
    f.close()


def generateOPERATORS_CUSTOM():
    qtdOperators = len(cfg.CUSTON_OPERATORS_NAME)
    cfg.CUSTOM_UNITARY = np.zeros(
        (cfg.STATESIZE, cfg.STATESIZE * qtdOperators), dtype=complex)

    for i in range(0, qtdOperators):
        currentLine = 0
        currenColumn = 0
        for line in open(cfg.CUSTON_OPERATORS_NAME[i], 'r'):

            if line == "\n":
                continue
            if line.startswith("#"):
                continue
            line = line.split()

            for j in range(0, 2 * cfg.STATESIZE, 2):
                cfg.CUSTOM_UNITARY[currentLine][
                    i * cfg.STATESIZE +
                    currenColumn] = float(line[j]) + 1J * float(line[j + 1])
                currenColumn = currenColumn + 1
            currenColumn = 0
            currentLine = currentLine + 1


#    print cfg.CUSTOM_UNITARY
#            op.check_Unitarity(cfg.COINOPERATOR,N)


def local_printS_2D(N):
    s = np.zeros([cfg.STATESIZE, cfg.STATESIZE], dtype=int)
    f = open("S.dat", "w")
    for k in range(4):
        for x in range(N):
            for y in range(N):
                if k == 0:
                    f.write("%d %d 1\n" % (T(
                        (x + 1) % N, y, iC(k), N) + 1, T(x, y, k, N) + 1))
                    s[T((x + 1) % N, y, iC(k), N)][T(x, y, k, N)] = 1
                elif k == 1:
                    f.write("%d %d 1\n" %
                            (T(x,
                               (y + 1) % N, iC(k), N) + 1, T(x, y, k, N) + 1))
                    s[T(x, (y + 1) % N, iC(k), N)][T(x, y, k, N)] = 1
                elif k == 2:
                    f.write("%d %d 1\n" % (T(
                        (x - 1) % N, y, iC(k), N) + 1, T(x, y, k, N) + 1))
                    s[T((x - 1) % N, y, iC(k), N)][T(x, y, k, N) + 1] = 1
                elif k == 3:
                    f.write("%d %d 1\n" %
                            (T(x,
                               (y - 1) % N, iC(k), N) + 1, T(x, y, k, N) + 1))
                    s[T(x, (y - 1) % N, iC(k), N)][T(x, y, k, N)] = 1

    return s


def printCtI_2D(N, Coin):
    f = open("CtI.dat", "w")
    for i in range(4):
        for j in range(4):
            for w in range(N * N):
                f.write("%d %d %f %f\n" %
                        (N * N * i + w + 1, N * N * j + w + 1, Coin[i][j].real,
                         Coin[i][j].imag))


D = 0
G = [[-0.5, 0.5, 0.5, 0.5], [0.5, -0.5, 0.5, 0.5], [0.5, 0.5, -0.5, 0.5],
     [0.5, 0.5, 0.5, -0.5]]
mI = [[-1, 0.0, 0.0, 0.0], [0.0, -1, 0.0, 0.0], [0.0, 0.0, -1, 0.0],
      [0.0, 0.0, 0.0, -1]]


def iC(k):
    return (k + 2) % 4


def T(x, y, k, N):
    if D == 1:
        return "(" + str(x) + "," + str(y) + ")"
    else:
        return k * N * N + x * N + y


def printD_2D(N):

    f = open("D.dat", "w")
    for x in range(N):
        for y in range(N):
            f.write("%f\n" % (np.sqrt(x * x + y * y)))


#
#def local_printD_2D( N ):
#    d=np.zeros([cfg.GRAPHSIZE,1],dtype=float)
#    f = open("D.dat", "w" )
#    i=0
#    for x in range( N ):
#        for y in range( N ):
#            f.write("%f\n"%(np.sqrt( x*x + y*y )))
#            d[i]=np.sqrt( x*x + y*y )
#            i=i+1
#    return d

#def printS_2D( N ):
#    f = open("S.dat", "w" )
#    for k in range( 4 ):
#        for x in range( N ):
#            for y in range( N ):
#                if k == 0:
#                    f.write("%d %d 1\n"%(T( (x + 1) % N, y, iC(k), N )+1, T( x, y, k, N )+1))
#                elif k == 1:
#                    f.write("%d %d 1\n"%(T( x, (y + 1) % N,iC(k), N )+1, T( x, y,k, N )+1))
#                elif k == 2:
#                    f.write("%d %d 1\n"%(T( (x-1) % N, y, iC(k),N )+1, T( x, y,k, N )+1))
#                elif k == 3:
#                    f.write("%d %d 1\n"%(T( x, (y - 1) % N,iC(k), N )+1, T( x, y,k, N )+1))
#########################################################################
#########################################################################
#########################################################################



__version__ = '2.0a6'

from sys import modules as sys_modules

__USING_PDB__ = 'pdb' in sys_modules
__GENERATING_DOCS__ = 'sphinx' in sys_modules
# ignores debug print messages when generating docs
__DEBUG__ = __debug__ and __USING_PDB__ and not __GENERATING_DOCS__

PYNEBLINA_IMPORT_ERROR_MSG =  (
    "Could not import pyneblina interface. "
    + "Do you have neblina-core and pyneblina installed?"
    + "\n#######################\n"
    + "Continuing without hpc."
    + "\n#######################\n" )



# -*- coding: utf-8 -*-
"""
Created on Wed Jul 30 00:44:17 2014

@author: aaron
"""
import config as cfg
import distance as dist
import numpy as np
import state as st
import operators as op
import standardDeviation as sta
import os
import os.path

def preParsing(inputFile):
    f=open(inputFile,'r')
    while True:
        line=f.readline()
        line=str.upper(line)
        if not line: break     ### EOF ###
        line=line.split()
        if line!='\n' and line:



            if line[0]=="WALK":
                if line.__len__()==1:
                    print("[HIPERWALK] Syntax error at  WALK BLOCK, missing type.")
                    exit(-1)
    
                elif (line[1] != "DTQW") and (line[1] != "STAGGERED") and (line[1] != "SZEGEDY") and (line[1] != "CUSTOM"):
                    print("[HIPERWALK] Syntax error at  WALK BLOCK, unknown walk type, %s."%line[1])
                    exit(-1)
                line[1]=str.upper(line[1])
                cfg.WALK=line[1]
    
        
            elif line[0]=="STEPS":
                if line.__len__()==1:
                    print("[HIPERWALK] Syntax error at STEPS BLOCK.")
                    exit(-1)
                cfg.STEPS=int(line[1])



            elif line[0]=="GRAPH":
                line[1]=str.upper(line[1])
                if line.__len__()==1:
                    print("[HIPERWALK] Syntax error at  GRAPH BLOCK, missing type.")
                    exit(-1)
                elif line[1]=="CYCLE":
                    if line.__len__()==2:
                        print("[HIPERWALK] Syntax error at  GRAPH BLOCK, missing number of sites.")
                        exit(-1)                
                    cfg.GRAPHTYPE="CYCLE"
                    cfg.GRAPHSIZE=int(line[2])
                    if cfg.WALK=="DTQW":                
                        cfg.WALK="DTQW1D"
                    elif cfg.WALK=="STAGGERED":
                        cfg.WALK="STAGGERED1D"
    
                elif line[1]=="LINE":
                    cfg.GRAPHSIZE=int(2*cfg.STEPS+1)
                    cfg.GRAPHTYPE="LINE"                
                    if cfg.WALK=="DTQW":                
                        cfg.WALK="DTQW1D"
                    elif cfg.WALK=="STAGGERED":
                        cfg.WALK="STAGGERED1D"
                
                elif line[1]=="TORUS":
                    if line.__len__()==2:
                        print("[HIPERWALK] Syntax error at  GRAPH BLOCK, missing size of TORUS.")
                        exit(-1)  
                    cfg.TORUSSIZE = [int(row)  for row in line[2:]]
                    cfg.GRAPHTYPE="TORUS"
                    if cfg.WALK=="DTQW":                
                        cfg.WALK="DTQW2D"
                    elif cfg.WALK=="STAGGERED":
                        cfg.WALK="STAGGERED2D"
    
                elif line[1]=="LATTICE":
                    cfg.TORUSSIZE = [int(row)  for row in line[2:]]
                    cfg.GRAPHTYPE="LATTICE"
                    if cfg.WALK=="DTQW":                
                        cfg.WALK="DTQW2D"
                    elif cfg.WALK=="STAGGERED":
                        cfg.WALK="STAGGERED2D"
    
                else:
                    print("[HIPERWALK] Syntax ERROR on GRAPH BLOCK, unknown type.")
                    exit(-1)



            elif line[0]=="GRAPHDIMENSION":
                if line.__len__()==1:
                    print("[HIPERWALK] Syntax error at  GRAPHDIMENSION BLOCK, missing value.")            
                if int(line[1])<= 0:
                    print("[HIPERWALK] Syntax error at  GRAPHDIMENSION BLOCK, value must be positive integer.")
                    exit(-1)
                else:
                    cfg.GRAPHDIMENSION=int(line[1])
                
                
            elif line[0]=="POLYGONS":
                if line.__len__()==1:
                    print("[HIPERWALK] Syntax error at POLYGONS BLOCK, missing values.")
                    exit(-1)
                elif cfg.WALK=="STAGGERED1D" and len(line)==2:
                    cfg.TESSELLATIONPOLYGONS=[int(line[1])]
                    cfg.NUMBER_OF_COEFICIENTS=int(line[1])
                
                elif cfg.WALK=="STAGGERED2D" and len(line)==3:
                    cfg.TESSELLATIONPOLYGONS=[int(line[1]),int(line[2])]
                    cfg.NUMBER_OF_COEFICIENTS=int(line[1])*int(line[2])
                else:
                    print("[HIPERWALK] Syntax error at TESSELLATIONPOLYGONS BLOCK, too many arguments.")
                    exit(-1)
            
def parsingFile(inputFile):

    f=open(inputFile,'r')
    while True:
        line=f.readline()

        if not line: break     ### EOF ###
        line=line.split()
        if line!='\n' and line:
            parsingLines(line,f)

def parsingLines(line,f):
        line[0]=str.upper(line[0])




        if line[0]=="LATTYPE":
            if line.__len__()==1:
                print("[HIPERWALK] Syntax error at  LATTYPE, missing lattice type.")
                exit(-1)       
            if (line[1] != "NATURAL") and (line[1] != "DIAGONAL") and (line[1] != "TEST"):
                print("[HIPERWALK] Syntax error at  LATTYPE, unknown lattice type, %s"%line[1])
                exit(-1)       
            cfg.LATTYPE=line[1]


     

        elif line[0]=="BEGINSTATE":
            cfg.STATE_COMPONENTS=[]
            totalComponents=0
            while line[0]!="ENDSTATE":

                line=f.readline()

                while line=="\n":     ### Parsing \n
                    line=f.readline()
                    
                line=str.upper(line)
                line=line.split()

                if line[0]!="ENDSTATE":
                    totalComponents=totalComponents+1
                    if cfg.WALK=="DTQW1D":                        
                        cfg.STATE_COMPONENTS=np.append(cfg.STATE_COMPONENTS,[float(line[0]),float(line[1]),int(line[2]),int(line[3])],0)
                    elif cfg.WALK=="DTQW2D":                        
                        cfg.STATE_COMPONENTS=np.append(cfg.STATE_COMPONENTS,[float(line[0]),float(line[1]),int(line[2]),int(line[3]),int(line[4])],0)                            
                    elif cfg.WALK=="STAGGERED1D": 
                        cfg.STATE_COMPONENTS=np.append(cfg.STATE_COMPONENTS,[float(line[0]),float(line[1]),int(line[2])],0)                            
                    elif cfg.WALK=="STAGGERED2D": 
                        cfg.STATE_COMPONENTS=np.append(cfg.STATE_COMPONENTS,[float(line[0]),float(line[1]),int(line[2]),int(line[3])],0)                        
        
            if cfg.WALK=="DTQW1D":
                cfg.STATE_COMPONENTS.shape=totalComponents,4
                aux=int((st.return_MAX_Position(cfg.STATE_COMPONENTS,2)+1))
                if aux==0:
                    aux=aux+2
                elif aux==1:
                    aux=aux+1
                cfg.COINVECTORDIMENSION=aux
                
            if cfg.WALK=="DTQW2D":
                cfg.STATE_COMPONENTS.shape=totalComponents,5
                cfg.COINVECTORDIMENSION=4

            if cfg.WALK=="STAGGERED1D":
                cfg.STATE_COMPONENTS.shape=totalComponents,3

            if cfg.WALK=="STAGGERED2D":
                cfg.STATE_COMPONENTS.shape=totalComponents,4
            st.generateState()



                
        elif line[0]=="BEGINCOIN" :
            line=f.readline()
            while line=="\n":     ### Parsing \n
                line=f.readline()                
            line=str.upper(line)
            line=line.split()
            
            if line[0]=="IDENTITY":
                cfg.COINOPERATORNAME="IDENTITY"
                cfg.COINOPERATOR=op.identity(int(line[1]))
            elif line[0]=="HADAMARD":
                cfg.COINOPERATORNAME="HADAMARD"
                cfg.COINOPERATOR=op.hadamard(int(line[1]))
            elif line[0]=="FOURIER":
                cfg.COINOPERATORNAME="FOURIER"
                cfg.COINOPERATOR=op.fourier(int(line[1]))
            elif line[0]=="GROVER":
                cfg.COINOPERATORNAME="GROVER"
                cfg.COINOPERATOR=op.grover(int(line[1]))
            else:
                N=len(line)/2
                cfg.COINOPERATOR=np.zeros((N,N),dtype=complex)
                for i in range(N):
                    aux=0
                    for j in range(0,2*N,2):
                        cfg.COINOPERATOR[i][aux]=float(line[j])+1J*float(line[j+1])
                        aux=aux+1
                    line=f.readline()
                    line=line.split()
                op.check_Unitarity(cfg.COINOPERATOR,N)
            



        elif line[0]=="BEGINTESSELLATION":
            totalComponents=0
            while line[0]!="ENDTESSELLATION":

                line=f.readline()
                while line=="\n":     ### Parsing \n
                    line=f.readline()
                line=str.upper(line)
                line=line.split()

                if line[0]!="ENDTESSELLATION":
                    if len(line)==2*cfg.NUMBER_OF_COEFICIENTS:
                        st.checkUnitarity(line)
                        cfg.STAGGERED_COEFICIENTS=np.append(cfg.STAGGERED_COEFICIENTS,[float(row)  for row in line],1)

                        line=f.readline()
                        line=str.upper(line)
                        line=line.split()
                        st.checkUnitarity(line)
                        if len(line)==2*cfg.NUMBER_OF_COEFICIENTS:
                            cfg.STAGGERED_COEFICIENTS=np.append(cfg.STAGGERED_COEFICIENTS,[float(row)  for row in line],1)
                        else:
                            print("[HIPERWALK] Syntax error at BEGINTESSELLATION BLOCK, invalid number of values.")
                            exit(-1)
                            
                        cfg.STAGGERED_COEFICIENTS.shape=2,cfg.NUMBER_OF_COEFICIENTS*2


                    else:
                        print("[HIPERWALK] Syntax error at BEGINTESSELLATION BLOCK, invalid number of values.")
                        exit(-1)




        elif line[0]=="DISPLACEMENT":
            if line.__len__()==1:
                print("[HIPERWALK] Syntax error at  DISPLACEMENT BLOCK, missing values.")
                exit(-1)
            elif cfg.WALK=="STAGGERED1D":
                cfg.TESSELLATIONDISPLACEMENT=[int(line[1])]
            elif cfg.WALK=="STAGGERED2D":
                cfg.TESSELLATIONDISPLACEMENT=[int(line[1]),int(line[2])]




        elif line[0]=="DIRECTORY":
            if line.__len__()==1:
                print("[HIPERWALK] Syntax error at  DIRECTORY BLOCK, missing name.")
                exit(-1)
            cfg.DIRECTORY=line[1]



        elif line[0]=="ANIMATION":
            if line.__len__()==1:
                print("[HIPERWALK] Syntax error at  ANIMATION BLOCK, missing name.")
                exit(-1)
            if line[1]=="TRUE":
                cfg.ANIMATION=1
            elif line[1]=="FALSE":
                cfg.ANIMATION=0
            else:
                print("[HIPERWALK] Syntax error at  ANIMATION BLOCK, unknown value, %s."%line[1])
                exit(-1)


        elif line[0]=="DELAY":
            if line.__len__()==1:
                print("[HIPERWALK] Syntax error at  DELAY BLOCK, missing value.")
                exit(-1)
            if int(line[1])<= 0:
                print("[HIPERWALK] Syntax error at  DELAY BLOCK, value must be positive integer.")
                exit(-1)
            cfg.DELAY=int(line[1])


        elif line[0]=="SIMULATION":
            line[1]=str.upper(line[1])
            if line.__len__()==1:
                print("[HIPERWALK] Syntax error at  SIMULATION BLOCK, missing simulation type.")
                exit(-1)
            if (line[1] != "LOCAL") and (line[1] != "PARALLEL"):
                print("[HIPERWALK] Syntax error at  SIMULATION BLOCK, unknown type. %s"%line[1])
                exit(-1)
            cfg.SIMULATIONTYPE=line[1]
            




        elif line[0]=="HARDWAREID":
                if line.__len__()==1:
                    print("[HIPERWALK] Syntax error at  HARDWAREID BLOCK, missing platform ID.")
                    exit(-1)
                cfg.HARDWAREID=int(line[1])

                
        elif line[0]=="PLOTTING":
            if line.__len__()==1:
                print("[HIPERWALK] Syntax error at  PLOTTING BLOCK, missing name.")
                exit(-1)
            line[1]=str.upper(line[1])                
            if line[1]=="ZEROS":
                cfg.PLOTTING_ZEROS=1
            
            
        elif line[0]=="PLOTS":
            if line.__len__()==1:
                print("[HIPERWALK] Syntax error at  PLOTS BLOCK, missing value.")            
                exit(-1)
            if line[1]=="TRUE":
                cfg.GNUPLOT=1
            elif line[1]=="FALSE":
                cfg.GNUPLOT=0
            else:
                print("[HIPERWALK] Syntax error at  PLOTS BLOCK, unknown value, %s."%line[1])
                exit(-1)


        elif line[0]=="VARIANCE":
            if line.__len__()==1:
                print("[HIPERWALK] Syntax error at  VARIANCE BLOCK, missing value.")            
                exit(-1)
            if line[1]=="TRUE":
                cfg.VARIANCE=1
            elif line[1]=="FALSE":
                cfg.VARIANCE=0
            else:
                print("[HIPERWALK] Syntax error at  VARIANCE BLOCK, unknown value, %s."%line[1])
                exit(-1)
       
       
        elif line[0]=="ALLSTATES":
            if line.__len__()==1:
                print("[HIPERWALK] Syntax error at  ALLSTATES BLOCK, missing value.")            
            if int(line[1])<= 0:
                print("[HIPERWALK] Syntax error at  ALLSTATES BLOCK, value must be positive integer.")
                exit(-1)
            else:
                cfg.ALLSTATES=1
                cfg.SAVE_STATES_MULTIPLE_OF_N=int(line[1])
        
        elif line[0]=="ALLPROBS":
            if line.__len__()==1:
                print("[HIPERWALK] Syntax error at  ALLPROBS BLOCK, missing value.")            
            if int(line[1])<= 0:
                print("[HIPERWALK] Syntax error at  ALLPROBS BLOCK, value must be positive integer.")
                exit(-1)
            else:
                cfg.ALLPROBS=1
                cfg.SAVE_PROBS_MULTIPLE_OF_N=int(line[1])
        elif line[0]=="INITIALSTATE":
            
            if line.__len__()==1:
                print("[HIPERWALK] Syntax error at  INITIALSTATE, missing filename.")
                exit(-1)

            if not os.path.isfile( line[1] ) :
                print("[HIPERWALK] Error at  INITIALSTATE, file '%s' not found."%(line[1]))
                exit(-1)
                
            a=str(os.path.abspath(str(line[1])))
            cfg.CUSTOM_INITIALSTATE_NAME=a
            st.generateState_CUSTOM()
            
        elif line[0]=="ADJMATRIX":
            if line.__len__()==1:
                print("[HIPERWALK] Syntax error at  ADJMATRIX, missing filename.")
                exit(-1)

            if not os.path.isfile( line[1] ) :
                print("[HIPERWALK] Error at  ADJMATRIX, file '%s' not found."%(line[1]))
                exit(-1)
                
            a=str(os.path.abspath(str(line[1])))
            cfg.ADJMATRIX_PATH=a
        

        elif line[0]=="OVERLAP":
            if line.__len__()==1:
                print("[HIPERWALK] Syntax error at  OVERLAP BLOCK, missing value.")            
                exit(-1)
            if line[1]=="TRUE":
                cfg.OVERLAP=1
            elif line[1]=="FALSE":
                cfg.OVERLAP=0
            else:
                print("[HIPERWALK] Syntax error at  OVERLAP BLOCK, unknown value, %s."%line[1])
                exit(-1)



        elif line[0]=="UNITARY":
             if line.__len__()==1:
                print("[HIPERWALK] Syntax error at  UNITARY, missing filename.")
                exit(-1)
             for i in range( 1, len( line) ):
                if( os.path.isfile( line[i] ) == False ):
                    print("[HIPERWALK] Error at  UNITARY, file '%s' not found."%(line[i]))
                    exit(-1)

                a=str(os.path.abspath(str(line[i])))
                cfg.CUSTON_OPERATORS_NAME.append( a )
                cfg.CUSTOM_UNITARY_COUNTER=len(line)-1

        elif line[0]=="LABELS":
             if line.__len__()==1:
                print("[HIPERWALK] Syntax error at  LABELS , missing filename.")
                exit(-1)
             for i in range( 1, len( line) ):
                if( os.path.isfile( line[i] ) == False ):
                    print("[HIPERWALK] Error at  LABELS, file '%s' not found."%(line[i]))
                    exit(-1)

                a=str(os.path.abspath(str(line[i])))
                cfg.CUSTOM_LABELS_NAME.append( a )
                cfg.CUSTOM_LABELS_NAME_COUNTER=len(line)-1
             sta.generateDistance_CUSTOM()


        elif line[0]=="DEBUG":
            cfg.DEBUG=1               



# -*- coding: utf-8 -*-
"""
Created on Mon Jul 21 05:27:04 2014

@author: aaron
"""


import numpy as np
import config as cfg
import os


def savetxt(string,array,arrayType,formatFloat):
    if arrayType == int:
        np.savetxt(str(string),array.view(int),fmt='%d')
    else:
        np.savetxt(str(string),array.view(float),fmt=formatFloat)
        

def initializeFiles():
    if cfg.WALK=="DTQW1D" or cfg.WALK=="STAGGERED1D":
        range1D()
        
    elif cfg.WALK=="DTQW2D" or cfg.WALK=="STAGGERED2D":
        range2D()
        distancesVector_2D()
        
def range1D():
    output = open("HIPERWALK_TEMP_RANGE_1D.dat",'w')    
       
    for x in  np.arange(cfg.RANGEX[0],cfg.RANGEX[1]+1,1):
        output.write("%d\n"%(x))
    output.close()


def range2D(): 
    output = open("HIPERWALK_TEMP_RANGE_2D.dat",'w')    
       
    for x in  np.arange(cfg.RANGEX[0],cfg.RANGEX[1]+1,1):
        for y in np.arange(cfg.RANGEY[0],cfg.RANGEY[1]+1,1):
            output.write("%d\t%d\t\n"%(x,y))
    output.close()
    
def distancesVector_2D():
    X = open("HIPERWALK_TEMP_X.dat", "w" ) 
    Y = open("HIPERWALK_TEMP_Y.dat", "w" ) 
  
    
    for x in range( cfg.RANGEX[0],cfg.RANGEX[1]+1 ):
        for y in range( cfg.RANGEY[0],cfg.RANGEY[1]+1 ):
            X.write("%d\n"%(x))
            Y.write("%d\n"%(y))
    X.close()
    Y.close()
    
def directory_Creation():
    cfg.OLD_DIRECTORY=os.getcwd() 
#    os.system("rm -rf *.pyc")
    if cfg.DIRECTORY!='' and cfg.DIRECTORY:
        
        if not os.path.exists(cfg.DIRECTORY):
            try:
                os.mkdir("%s"%cfg.DIRECTORY)
            except OSError: # this would be "except OSError, e:" before Python 2.6
                print("[HIPERWALK] Could not creat folder %s."%cfg.DIRECTORY)
                exit(-2)

#            os.mkdir("%s"%cfg.DIRECTORY)

        if cfg.WALK=="DTQW1D":
            os.system("cp %s/hiperwalk/dtqw1d.nbl %s/HIPERWALK_TEMP_DTQW1D.nbl"%(cfg.INSTALL_DIR, cfg.DIRECTORY))
        elif cfg.WALK=="DTQW2D":
            os.system("cp %s/hiperwalk/dtqw2d.nbl %s/HIPERWALK_TEMP_DTQW2D.nbl"%(cfg.INSTALL_DIR, cfg.DIRECTORY))
        elif cfg.WALK=="STAGGERED1D":
            os.system("cp %s/hiperwalk/staggered1d.nbl %s/HIPERWALK_TEMP_STAGGERED1D.nbl"%(cfg.INSTALL_DIR, cfg.DIRECTORY))
        elif cfg.WALK=="STAGGERED2D":
            os.system("cp %s/hiperwalk/staggered2d.nbl %s/HIPERWALK_TEMP_STAGGERED2D.nbl"%(cfg.INSTALL_DIR, cfg.DIRECTORY))
        elif cfg.WALK=="CUSTOM":
            os.system("cp %s/hiperwalk/custom.nbl %s/HIPERWALK_TEMP_CUSTOM.nbl"%(cfg.INSTALL_DIR, cfg.DIRECTORY))
        os.chdir("%s"%cfg.DIRECTORY)
        
        
    else:
        print("[HIPERWALK] Error at DIRECTORY BLOCK, missing value!")
        exit(-1)
        
def cleaningTemporaryFiles():
    os.system("rm -rf HIPERWALK_TEMP*")
    os.system("rm -rf NEBLINA_TEMP*")

    if not cfg.TEST_MODE:
        os.chdir(cfg.OLD_DIRECTORY)

def remove(filename):
    try:
        os.remove(filename)
    except OSError: # this would be "except OSError, e:" before Python 2.6
        pass

    
def test_mode():
#    a=str(os.getcwd())
#    print("Teste %s"%a)

    os.chdir(os.environ['HOME'])
    if not os.path.exists("HIPERWALK_TEST_DIRECTORY"):
        os.mkdir("HIPERWALK_TEST_DIRECTORY")
#    os.system("cp *.in HIPERWALK_TEMP_DIRECTORY")
#    os.system("cp psi0.dat HIPERWALK_TEMP_DIRECTORY")
#    os.system("cp u0.dat HIPERWALK_TEMP_DIRECTORY")
#    os.system("cp u1.dat HIPERWALK_TEMP_DIRECTORY")
    os.chdir("HIPERWALK_TEST_DIRECTORY")


def remnove_test_mode_folder():
    os.chdir(os.environ['HOME'])
    os.system("rm -rf HIPERWALK_TEST_DIRECTORY")



# -*- coding: utf-8 -*-
"""
Created on Mon Jan 19 16:45:09 2015

@author: Aaron Leao
         aaron@lncc.br
"""
import operators as op
import config as cfg
import ioFunctions as io 
import neblina as nb
import gnuplot as gnuplot
import testmode
import numpy as np
def run():
    
    if not cfg.OVERLAP:
        cfg.OVERLAPX=int(cfg.TESSELLATIONPOLYGONS[0])
        cfg.OVERLAPY=int(cfg.TESSELLATIONPOLYGONS[1])


    op.STAGGERED2D()


#    i1=np.dot(cfg.Ueven,np.conjugate(cfg.Ueven))
#    i2=np.dot(cfg.Uodd,np.conjugate(cfg.Uodd))
#    io.savetxt("A.dat",i1,float,'%1.1f')
#    io.savetxt("B.dat",i2,float,'%1.1f')
    
    cfg.DISTANCE_VECTOR_SIZE=cfg.GRAPHSIZE
    
    io.savetxt("HIPERWALK_TEMP_PSI.dat",cfg.STATE,float,'%1.16f')

    nb.runCore_STAGGERED2D()
    
    cfg.STATE=nb.neblina_state_to_vector("NEBLINA_TEMP_final_state.dat")
    probabilities=nb.neblina_distribution_to_vector("NEBLINA_TEMP_final_distribution.dat")

    output = open("final_distribution.dat",'w')
    output1 = open("final_state.dat",'w')
    output.write("POS X \t POS Y \t PROBABILITY\n")          
    output1.write("POS X \t POS Y \t Re(amplitude)\t \t Im(amplitude)\n")   



    if cfg.GRAPHTYPE=="TORUS":
        for x in range(cfg.RANGEX[0],cfg.RANGEX[1],1):
            for y in range(cfg.RANGEY[0],cfg.RANGEY[1],1):
                index=(x)*(cfg.RANGEX[1])+(y)
#                print index, x, y
                output.write("%d\t %d\t %1.16f\n"%(x,y,probabilities[index]))
                output1.write("%d\t %d\t %1.16f \t %1.16f\n"%(x,y,cfg.STATE[index].real,cfg.STATE[index].imag))         
        output.close()
        output1.close()

    elif cfg.GRAPHTYPE=="LATTICE":
        for x in range(cfg.RANGEX[0],cfg.RANGEX[1]+1,1):
            for y in range(cfg.RANGEY[0],cfg.RANGEY[1]+1,1):
                index=(x-cfg.RANGEX[0])*cfg.SIZEY+(y-cfg.RANGEY[0])    
                output.write("%d\t %d\t %1.16f\n"%(x,y,probabilities[index]))
                output1.write("%d\t %d\t %1.16f \t %1.16f\n"%(x,y,cfg.STATE[index].real,cfg.STATE[index].imag))         
        output.close()
        output1.close()

    if cfg.GNUPLOT:
        io.savetxt("HIPERWALK_TEMP_PROBABILITIES",probabilities,float,'%1.16f')
        gnuplot.template_DTQW2D("HIPERWALK_TEMP_PROBABILITIES","final_distribution.eps","EPS")

        if cfg.STEPS>1:
            gnuplot.plotStatistics2d()
            
        if cfg.ANIMATION == 1:
            gnuplot.plotAnimation2D()


    if cfg.TEST_MODE:
        modelVector=testmode.create_STAGGERED2D_test_vector()
        returnNeblina=nb.neblina_distribution_to_vector("NEBLINA_TEMP_final_distribution.dat")
        if np.linalg.norm(modelVector-returnNeblina,np.inf) == float(0):
            return 1
        else:
            return 0
                
    return 1



import numpy as np
import scipy.sparse
from .graph import Graph
from warnings import warn

class Hypercube(Graph):
    r"""
    Hypercube graph.

    The hypercube graph consists of ``2**n`` vertices,
    where ``n`` is the hypercube *dimension*.
    The numerical labels of these vertices  are
    ``0``, ``1``, ..., ``2**n - 1``.
    Two vertices are adjacent
    if and only if the corresponding binary tuples
    differ by only one bit, indicating a Hamming distance of 1.

    Parameters
    ----------
    dimension : int
        The dimension of the hypercube.

    Notes
    -----
    A vertex :math:`v` in the hypercube is adjacent to all other vertices
    that have a Hamming distance of 1. To put it differently, :math:`v`
    is adjacent to :math:`v \oplus 2^0`, :math:`v \oplus 2^1`,
    :math:`\ldots`, :math:`v \oplus 2^{n - 2}`, and
    :math:`v \oplus 2^{n - 1}`.
    Here, :math:`\oplus` represents the bitwise XOR operation,
    and :math:`n` signifies the dimension of the hypercube.

    The order of the arcs is determined by the XOR operation.
    Consider two arcs, :math:`(u, u \oplus 2^i)` and :math:`(v, v \oplus 2^j)`,
    labeled numerically as :math:`a_1` and :math:`a_2`, respectively.
    The condition :math:`a_1 < a_2` is true if and only
    if either :math:`u < v` is true, or
    both :math:`u = v` and :math:`i < j` are true.
    """
    def __init__(self, dimension):
        num_vert = 1 << dimension
        num_arcs = dimension*num_vert

        data = np.ones(num_arcs, dtype=np.int8)
        indptr = np.arange(0, num_arcs + 1, dimension)
        indices = np.array([v ^ 1 << shift for v in range(num_vert)
                                           for shift in range(dimension)])
        # for v in range(num_vert):
        #     indices[v*dimension:(v + 1)*dimension].sort()

        adj_matrix = scipy.sparse.csr_array((data, indices, indptr),
                                            shape=(num_vert, num_vert))

        super().__init__(adj_matrix)
        self._dimension = int(dimension)

    def embeddable(self):
        return False

    def arc_direction(self, arc):
        r"""
        Returns the arc direction.

        The arc direction of ``(tail, head)`` is the number ``i``
        such that ``tail == head ^ 2**i``.

        Parameters
        ----------
        arc
            The arc can be represented in either arc notation (``(tail, head)``)
            or by using the numerical label  (``int``).

        Returns
        -------
        int
        """
        try:
            tail, head = arc
        except TypeError:
            tail, head = self.arc(arc)

        direction = tail ^ head
        try:
            # python >= 3.10
            count = direction.bit_count()
        except:
            count = bin(direction).count('1')

        try:
            direction = direction.bit_length() - 1
        except:
            direction = np.log2(direction)

        if count != 1 or direction < 0 or direction >= self._dimension:
            raise ValueError("Arc " + str(arc) + " does not exist.")

        return direction

    def arc_label(self, tail, head):
        direction = self.arc_direction((tail, head))
        return tail*self._dimension + direction

    def arc(self, label):
        tail = label // self._dimension
        direction = label - tail*self._dimension
        head = tail ^ (1 << direction)
        return (tail, head)

    def degree(self, vertex):
        return self._dimension

    def dimension(self):
        r"""
        Hypercube dimension.

        Returns
        -------
        int
        """
        return self._dimension



# -*- coding: utf-8 -*-
"""
Created on Wed Jul 30 00:47:37 2014

@author: aaron
Module containing the global variables.
"""
import numpy as np
import config as cfg


def reset():
    ##
    ##  General Variables
    ##
    cfg.WALK = ""
    cfg.DIRECTORY = "HIPERWALK_DEFAULT_FOLDER"
    cfg.STEPS = 0
    cfg.GRAPHSIZE = 0
    cfg.STATE_COMPONENTS = []
    cfg.STATE = []
    cfg.STATESIZE = 0
    cfg.SIMULATIONTYPE = "PARALLEL"
    cfg.GRAPHTYPE = ""
    cfg.SIZEX = 0
    cfg.SIZEY = 0
    cfg.RANGEX = []
    cfg.RANGEY = []
    cfg.COINOPERATORNAME = ""
    cfg.ALLSTATES = 0
    cfg.SAVE_STATES_MULTIPLE_OF_N = 0
    cfg.ALLPROBS = 0
    cfg.SAVE_PROBS_MULTIPLE_OF_N = 0

    ##
    ##  Statistics
    ##
    cfg.DISTANCES_VECTOR = []
    cfg.DISTANCES_SQUARE = []
    cfg.DISTANCES_VECTOR_X = []
    cfg.DISTANCES_VECTOR_Y = []
    cfg.DISTANCES_VECTOR_SQUARE_X = []
    cfg.DISTANCES_VECTOR_SQUARE_Y = []

    ##
    ##  Coined Walk
    ##
    cfg.COINVECTOR = []
    cfg.COINOPERATOR = np.array([], dtype=complex)
    cfg.SHIFTOPERATOR = []
    cfg.COINTENSORIDENTITY = []
    cfg.COINVECTORDIMENSION = 0

    ###
    ### DTQW2D
    ###
    cfg.LATTYPE = "DIAGONAL"
    cfg.TORUSSIZE = np.array([], dtype=int)

    ###  STAGGERED Walk
    ###
    cfg.Ueven = []
    cfg.Uodd = []
    cfg.STAGGERED_COEFICIENTS = []
    cfg.TESSELLATIONPOLYGONS = []
    cfg.TOTAL_PATCHES_IN_X = 0
    cfg.TOTAL_PATCHES_IN_Y = 0
    cfg.OVERLAP = 0
    cfg.OVERLAPX = 0
    cfg.OVERLAPY = 0
    cfg.NUMBER_OF_COEFICIENTS = 0
    ###
    ### Custom Walk
    ###
    cfg.CUSTON_OPERATORS_NAME = []
    cfg.CUSTOM_INITIALSTATE_NAME = []
    cfg.CUSTOM_UNITARY_COUNTER = 0
    cfg.CUSTOM_LABELS_NAME = []
    cfg.CUSTOM_LABELS_NAME_COUNTER = 0
    cfg.GRAPHDIMENSION = 1
    cfg.ADJMATRIX_PATH = "@NON_INICIALIZED@"

    ###
    ###   Neblina
    ###
    cfg.HARDWAREID = 0

    ###
    ###   Gnuplot
    ###
    cfg.GNUPLOT = 0
    cfg.ANIMATION = 0
    cfg.PLOTTING_ZEROS = 0
    cfg.DELAY = 20

    ###
    ###   Debug
    ###
    cfg.DEBUG = 0
    cfg.QWALK_RETURN = []
    cfg.RETURN_SIMULATION_FLAG = 1
    #    cfg.TEST_MODE=0
    cfg.OLD_DIRECTORY = ""

    cfg.NUMBER_OF_TESSELLATIONS = 0
    cfg.TESSELLATIONGEOMETRY = []
    cfg.TESSELLATIONDISPLACEMENT = []
    cfg.TESSELLATIONINITIALDISPLACEMENT = []
    cfg.TESSELLATIONCOEFICIENTS = []

    cfg.VARIANCE = 0


##
##  General Variables
##
INSTALL_DIR = "/usr/local"
WALK = ""
DIRECTORY = "HIPERWALK_DEFAULT_FOLDER"
STEPS = 0
GRAPHSIZE = 0
STATE_COMPONENTS = []
STATE = []
STATESIZE = 0
SIMULATIONTYPE = "PARALLEL"
GRAPHTYPE = ""
SIZEX = 0
SIZEY = 0
RANGEX = []
RANGEY = []
COINOPERATORNAME = ""
ALLSTATES = 0
SAVE_STATES_MULTIPLE_OF_N = 0
ALLPROBS = 0
SAVE_PROBS_MULTIPLE_OF_N = 0

##
##  Statistics
##
DISTANCES_VECTOR = []
DISTANCES_SQUARE = []
DISTANCES_VECTOR_X = []
DISTANCES_VECTOR_Y = []
DISTANCES_VECTOR_SQUARE_X = []
DISTANCES_VECTOR_SQUARE_Y = []

##
##  Coined Walk
##
COINVECTOR = []
COINOPERATOR = np.array([], dtype=complex)
SHIFTOPERATOR = []
COINTENSORIDENTITY = []
COINVECTORDIMENSION = 0

###
### DTQW2D
###
LATTYPE = "DIAGONAL"
TORUSSIZE = np.array([], dtype=int)

###  Coinless Walk
###
Ueven = []
Uodd = []
STAGGERED_COEFICIENTS = []
TESSELLATIONPOLYGONS = []
TOTAL_PATCHES_IN_X = 0
TOTAL_PATCHES_IN_Y = 0
OVERLAP = 0
OVERLAPX = 0
OVERLAPY = 0
NUMBER_OF_COEFICIENTS = 0
###
### Custom Walk
###
CUSTON_OPERATORS_NAME = []
CUSTOM_INITIALSTATE_NAME = []
CUSTOM_UNITARY_COUNTER = 0
CUSTOM_LABELS_NAME = []
CUSTOM_LABELS_NAME_COUNTER = 0
GRAPHDIMENSION = 1
ADJMATRIX_PATH = "@NON_INICIALIZED@"
###
###   Neblina
###
HARDWAREID = 0

###
###   Gnuplot
###
GNUPLOT = 0
ANIMATION = 0
PLOTTING_ZEROS = 0
DELAY = 20

###
###   Debug
###
DEBUG = 0
QWALK_RETURN = []
RETURN_SIMULATION_FLAG = 1
TEST_MODE = 0
OLD_DIRECTORY = ""

###
###  Novo Coinless
###
NUMBER_OF_TESSELLATIONS = 0
TESSELLATIONGEOMETRY = []
TESSELLATIONDISPLACEMENT = []
TESSELLATIONINITIALDISPLACEMENT = []
TESSELLATIONCOEFICIENTS = []

VARIANCE = 0



"""
Encapsules Animation class, responsible for managing animations.
"""

import matplotlib.pyplot as plt
from PIL import Image
from .._constants import __DEBUG__

if __DEBUG__:
    from time import time
    start = time()

class Animation:
    """
    Class responsible for creating, managing and saving animations.
    
    Attributes
    ----------
    frames : list
        List of frames.
    plt_anim : :class:`matplotlib.animation.FuncAnimation`
        Animation object of matplotlib.
    save_path : str
        Filepath for saving the animation.
    """

    def __init__(self):
        self.frames = []
        self.plt_anim = None
        self.save_path = None

    def __del__(self):
        def _delete(obj):
            if obj is not None:
                del obj

        _delete(self.plt_anim)
        _delete(self.frames)
        _delete(self.save_path)

    def add_frame(self, fig):
        """
        Draws and adds (appends) a new frame to the `frames` list.

        Parameters
        ----------
        fig : :class:`matplotlib.figure.Figure`
            Figure to be added.

        Examples
        --------
        .. todo::
            Create example
        """
        fig.canvas.draw()

        img = Image.frombytes('RGB', fig.canvas.get_width_height(),
            fig.canvas.tostring_rgb())
        self.frames.append(img)

        plt.close()

    def create_animation(self, interval):
        """
        Creates animation using the `frames` content.

        Parameters
        ----------
        interval : int
            Delay between frames in milliseconds.

        See Also
        --------
        add_frame

        Notes
        -----
        The animation is saved in `plt_anim` as a
        :class:`matplotlib.animation.FuncAnimation` object.

        .. todo::
            - Default values for `inteval`
            - Implement `repeat_delay` parameter.
                For matplotlib 3.5, saving the gif using
                either Pillow or ffmpeg
                ignores the `repeat_delay` parameter.
        """
        from ._plot import _configure_figure
        from matplotlib.animation import FuncAnimation

        fig, ax = _configure_figure(None)

        # It would be more efficient to use FuncAnimation
        # as a previous version;
        # i.e. to return only the artist that need to be redrawn.
        # But workarounds for updating colorbar ticks
        # and y-axes were not found.
        # TODO: documentation: recommend saving gif
        # and not showing for any animations
        def update_figure(img):
            ax.imshow(img)

            if __DEBUG__:
                global start
                end = time()
                start = end

        # TODO: repeat_delay not implemented in animation.save
        self.plt_anim = FuncAnimation(
            fig, update_figure, frames=self.frames,
            interval=interval, repeat=self._is_in_notebook()
        )
        # repeat=True causes updateFigure to be called
        # even when it is not needed.
        # However, it is necessary for creating a
        # looping video in jupyter notebook.

        # removing axes and pads introduced by imshow,
        # i.e. shows only the original axes and pads (from plots_as_imgs)
        plt.axis('off')
        plt.tight_layout(pad=0)
        if __DEBUG__:
            # used to check if no extra padding is being added
            fig.patch.set_facecolor('red')

    # TODO: saving video is supported but not recommended because
    #   fps may not be respected.
    # ffmpeg is necessary for saving video and better quality gifs.
    # Pillow is sufficient for saving gifs,
    # although colorbar may be discretized.
    def save_animation(self, filename):
        """
        Saves animation as a gif if it is already created.

        Parameters
        ----------
        filename : str
            Filename (with no format) where the animation will be saved.

        See Also
        --------
        create_animation

        Notes
        -----
        `repeat_delay` is not being used.
        For matplotlib 3.5, saving the gif using either Pillow or ffmpeg
        ignores the `repeat_delay` parameter.

        .. todo::
            Option to save animation as either gif or mp4 file
            (and potentially other formats).

        Examples
        --------
        >>> # add previous code
        >>> # saves animation in ``output.gif`` file
        >>> anim.save_animation('output') # doctest: +SKIP

        .. todo::
            Complete example
        """
        # TODO: add other valid matplotlib formats
        valid_extensions = ['.gif', '.mp4']

        extension = filename[-4:]
        if extension not in valid_extensions:
            filename += '.gif'
        self.save_path = filename

        self.plt_anim.save(self.save_path)
        plt.close()

        # by ignoring this condition while using the terminal,
        # a non-aborting exception is thrown:
        # AttributeError: 'NoneType' object has no attribute 'add_callback'.
        # The attributes deleted are needed for saving
        # html5 video in jupyter notebooks
        if not self._is_in_notebook():
            del self.plt_anim
            self.plt_anim = None
            del self.frames
            self.frames = None

    def _is_saved(self):
        return self.save_path != None

    # returns None if animation is already saved.
    # Otherwise returns the temporary file
    def _save_animation_in_temp_file(self):
        if not self._is_saved():
            import tempfile
            temp = tempfile.NamedTemporaryFile(suffix='.gif')
            self.save_animation(temp.name)
            return temp

        return None

    def _is_in_notebook(self):
        try:
            shell = get_ipython().__class__.__name__
            if shell == 'ZMQInteractiveShell':
                return True
            return False
        except:
            return False

    def show_animation(self):
        """
        Shows animation as long as it has been created.

        In Jupyter Notebook, use ``%matplotlib inline``
        before importing :obj:`PlotModule`.

        See Also
        --------
        create_animation

        Warnings
        --------
        The current version only supports GTK 3.0 (used in Ubuntu 20.04).
        It mus be updated to support GTK 4.0 (used in Ubuntu 22.04).

        Notes
        -----
        If the animation was saved, the file is open and shown.
        If it was not saved, it is stored in a temporary file
        and then shown.

        .. todo::

            Implement usage of GTK 4.0.
        """
        if self._is_in_notebook():
            self._show_animation_notebook()
        else:
            self._show_animation_terminal()

    def _show_animation_terminal(self):
        temp = self._save_animation_in_temp_file()

        from gi import require_version
        try:
            require_version('Gtk', '3.0')
        except ValueError:
            # Namespace Gtk is already loaded with version 4.0
            # detect which version was loaded
            pass

        from gi.repository import Gtk
        from gi.repository.Gdk import KEY_q
        from gi.repository.Gdk import KEY_Q
        from gi.repository.Gdk import RGBA
        
        Gtk_version = Gtk.get_major_version()
        
        if Gtk_version == 3:
            # creating window
            window = Gtk.ApplicationWindow(title="Animation")
            state = window.get_state()
            raise NotImplementedError(
                "window.get_state() is deprecated. Change."
            )
            window.override_background_color(state, RGBA(1, 1, 1, 1))

            def configure_gif_window(self):
                # assign closing events
                def on_key_press(self, event):
                    if event.keyval == KEY_q or event.keyval == KEY_Q :
                        window.close()

                window.connect("key-press-event", on_key_press)
                window.connect("destroy", Gtk.main_quit)

                # exhibits animation in Gtk window
                img = Gtk.Image.new()
                img.set_from_file(self.save_path)
                window.add(img)

            def configure_video_window(self):
                raise NotImplementedError('Showing video not supported.')

            if self.save_path[-4:] == '.gif':
                configure_gif_window(self)
            else:
                configure_video_window(self)


            # showing window and starting Gtk main loop
            window.show_all()
            Gtk.main()
            
            if temp is not None:
                self.save_path = None
                temp.close()

        elif Gtk_version == 4:
            # creating window
            window = Gtk.ApplicationWindow(title="Animation")
            state = window.get_state_flags()
            # window.override_background_color(state, RGBA(1, 1, 1, 1))

            def configure_gif_window(self):
                pass

            def configure_video_window(self):
                pass

            if self.save_path[-4:] == '.gif':
                configure_gif_window(self)
            else:
                configure_video_window(self)

            def on_activate(app, save_path):
                win = Gtk.ApplicationWindow(application=app)
                ############################
                from gi.repository import GdkPixbuf
                pixbuf = GdkPixbuf.PixbufAnimation.new_from_file(save_path)
                img = Gtk.Image()
                img.set_from_animation(pixbuf)
                win.set_child(img)
                win.present()
                ############################
                #img = Gtk.Image.new_from_file(save_path)
                #win.set_child(img)
                #win.present()
                ############################
                #img = Gtk.Video.new_for_filename(save_path)
                # img = Gtk.Image.new()
                # Gtk.Image.set_from_file(img, save_path)
                #img.set_from_file(save_path)
                ############################
                #stream = Gtk.MediaFile.new_for_filename(save_path)
                #video = Gtk.Video.new_for_media_stream(stream)
                #video.set_autoplay(True)
                #video.set_hexpand(True)
                #video.set_vexpand(True)
                #win.set_child(video)
                #win.present() 

            app = Gtk.Application()
            app.connect('activate', on_activate, self.save_path)
            app.run(None)
            # showing window and starting Gtk main loop
            # window.show_all()
            # Gtk.main()
            
            if temp is not None:
                self.save_path = None
                temp.close()

        else:
            raise ValueError(
                "GTK version not supported."
            )

    def _show_animation_notebook(self):
        from IPython import display

        # embedding animation in jupyter notebook
        video = self.plt_anim.to_html5_video()
        html = display.HTML(video)
        display.display(html)

        plt.close()

